/**
 * User: progsdi
 * Date: 16.04.12
 * Time: 13:37
 */

/* Dependencies */
/* check for required dependencies and break javascript if anything is missing */
/* break by exception with human-readable message */
var missing = "";
if (!paper) {
    /* paper.js - drawing framework */
    missing += "paper.js framework\n";
}
if (!$) {
    /* jquery.js - core javascript framework */
    missing += "jquery.js framework\n";
}
/*if (!???) {
    /* jquery-ui.js - ui framework upon jquery.js * /
    missing += "jquery-ui.js extension framework\n";
}*/
if (!$ || !$.layout) {
    /* jquery.layout.js - layout plugin upon jquery-ui.js */
    missing += "jquery.layout.js plugin\n";
}
if (!$ || !$("body").tree) {
    /* tree.jquery.js - tree view plugin upon jquery-ui.js */
    missing += "tree.jquery.js plugin\n";
}
if (!$ || !$.fn.jScrollPane) {
    /* jquery.jscrollpane.js - stylish scrolls plugin upon jquery-ui.js */
    missing += "jquery.jscrollpane.js UI extension\n";
}
if (!qq || !qq.FileUploader) {
    /* fileuploader.js - fileupload library */
    missing += "fileuploader.js library\n";
}
if (!diff_match_patch) {
    /* diff_match_patch.js - google diff library */
    missing += "diff_match_patch.js util library\n";
}
if (missing.length > 0) {
    throw new Error("Missing dependencies on:\n" + missing, 9);
}

/* Core encapsulating object */
var Desk = {};

/**initialize work desk of IDE
 * @param jqElement - element to insert work desk into
 */
Desk.init = function(jqElement) {
    /* Calls to server */
    /** method collection */
    var DeskApi = {};
    DeskApi.dummyTranslator = function(data) { return data; };
    DeskApi.fileTranslator = function(data) { return {success: true, data: data}; };
    DeskApi.jsonTranslator = function(data) { return $.parseJSON(data); };
    $.ajaxSetup({dataType: "text"});
    DeskApi.request = function(request, resultTranslator, callback, callbackObj, errorCallback) {
        if (typeof errorCallback != "function") {
            errorCallback = function(e) {
                if(console) { console.log(e); }
            }
        }
        if (typeof callback != "function") {
            callback = function() {};
        }
        request
            .success(function(data) {
                data = resultTranslator(data);
                if (!data.success) {
                    errorCallback.call(callbackObj, data.error);
                } else {
                    callback.call(callbackObj, data.data);
                }
            })
            .error(function(data) {
                errorCallback.call(callbackObj, data);
            });
    };
    DeskApi.get = function(url, params, callback, callbackObj, errorCallback) {
        DeskApi.request($.getJSON(url, params), DeskApi.dummyTranslator, callback, callbackObj, errorCallback);
    };
    DeskApi.getText = function(url, params, callback, callbackObj, errorCallback) {
        DeskApi.request($.get(url, params), DeskApi.fileTranslator, callback, callbackObj, errorCallback);
    };
    DeskApi.post = function(url, params, postParams, callback, callbackObj, errorCallback) {
        for(var k in params) {
            if (url.indexOf("?") >= 0) {
                url += "&";
            } else {
                url += "?";
            }
            url += k + "=" + encodeURIComponent(params[k]);
        }
        DeskApi.request($.post(url, postParams), DeskApi.jsonTranslator, callback, callbackObj, errorCallback);
    };
    DeskApi.users = {};
    DeskApi.users.current = function(callback, callbackObj, errorCallback) {
        DeskApi.get('/api/users/current/', undefined, callback, callbackObj, errorCallback);
    };
    DeskApi.files = {};
    DeskApi.files.list = function(projectName, branch, state, callback, callbackObj, errorCallback) {
        var params = {};
        if (branch) { params["branch"] = branch; }
        if (state) { params["state_id"] = state; }
        DeskApi.get('/api/files/list/' + projectName + '/', params, callback, callbackObj, errorCallback);
    };
    DeskApi.files.add = function(projectName, branch, state, category, name, type, content, callback, callbackObj, errorCallback) {
        var params = {name: name, type: type, category: category};
        if (branch) { params["branch"] = branch; }
        if (state) { params["state_id"] = state; }
        DeskApi.post('/api/files/add/' + projectName + '/', params, {content: content}, callback, callbackObj, errorCallback);
    };
    DeskApi.files.get = function(projectName, file, callback, callbackObj, errorCallback) {
        DeskApi.getText('/api/files/browse/' + projectName + '/' + (file.category && file.category.length ? file.category + '/' : '') + file.name + (file.type.extension.length ? '.' + file.type.extension : ''), undefined, callback, callbackObj, errorCallback);
    };
    DeskApi.files.find = function(name, type, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/files/find/', {name: name, type: type}, callback, callbackObj, errorCallback);
    };
    DeskApi.files.remove = function(projectName, branch, state, fileId, callback, callbackObj, errorCallback) {
        var params = {file_id: fileId};
        if (branch) { params["branch"] = branch; }
        if (state) { params["state_id"] = state; }
        DeskApi.get('/api/files/delete/' + projectName + '/', params, callback, callbackObj, errorCallback);
    };
    DeskApi.files.edit = function(projectName, branch, state, fileId, modification, callback, callbackObj, errorCallback) {
        var params = {file_id: fileId};
        if (branch) { params["branch"] = branch; }
        if (state) { params["state_id"] = state; }
        var m = {file: {
                        id: modification.file.id
                    },
                type: modification.type,
                start: modification.start,
                length: modification.length,
                newValue: modification.newValue
            };
        DeskApi.post('/api/files/edit/' + projectName + '/', params, {modification: JSON.stringify(m)}, callback, callbackObj, errorCallback);
    };
    DeskApi.files.editor = function(fileType, callback, callbackObj) {
        var res = {
            factories: []
        };
        res.callbackName = 'flightprocess_file_editor_factory_' + fileType + '_' + new Date().getTime();
        window[res.callbackName] = function(callbackName, loadDescriptor, nextCallback, nextCallbackObj) {
            return function(factories) {
                loadDescriptor.factories = factories;
                delete window[callbackName];
                nextCallback.call(nextCallbackObj, factories, loadDescriptor);
            };
        }(res.callbackName, res, callback, callbackObj);
        res.callback = window[res.callbackName];
        res.element = document.createElement( 'script' );
        res.element.type = 'text/javascript';
        res.element.src = "/api/files/editor/" + fileType + "/" + res.callbackName;
        $("head")[0].appendChild(res.element);
        return res;
    };
    DeskApi.projects = {};
    DeskApi.projects.list = function(callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/list', undefined, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.add = function(name, type, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/add', {name: name, type: type}, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.find = function(name, type, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/find', {name: name, type: type}, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.remove = function(projectName, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/delete/' + projectName, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.exclude = function(projectName, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/exclude/' + projectName, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.setProperties = function(projectName, properties, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/projects/set_properties/' + projectName, properties, callback, callbackObj, errorCallback);
    };
    DeskApi.projects.renderer = function(projectType, callback, callbackObj) {
        var res = {
            factories: []
        };
        res.callbackName = 'flightprocess_project_renderer_factory_' + projectType + '_' + new Date().getTime();
        window[res.callbackName] = function(callbackName, loadDescriptor, nextCallback, nextCallbackObj) {
            return function(factories) {
                loadDescriptor.factories = factories;
                delete window[callbackName];
                nextCallback.call(nextCallbackObj, factories, loadDescriptor);
            };
        }(res.callbackName, res, callback, callbackObj);
        res.callback = window[res.callbackName];
        res.element = document.createElement( 'script' );
        res.element.type = 'text/javascript';
        res.element.src = "/api/projects/renderer/" + projectType + "/" + res.callbackName;
        $("head")[0].appendChild(res.element);
        return res;
    };
    DeskApi.projectTypes = {};
    DeskApi.projectTypes.list = function(callback, callbackObj, errorCallback) {
        DeskApi.get('/api/project_types/list', undefined, callback, callbackObj, errorCallback);
    };
    DeskApi.projectTypes.listFiles = function(projectTypeId, callback, callbackObj, errorCallback) {
        DeskApi.get('/api/project_types/list_files/' + projectTypeId, undefined, callback, callbackObj, errorCallback);
    };

    /* On top floating panel with LOADING image */
    /** create LOADING panel */
    var DeskLoading = function() {
        /** inserted element with animating LOADING inside */
        this.animation = $("<div style='width:100%; height:100%; position:fixed; z-index:1500;'></div>");
        this.animation.hide();
        var loading = "<table style='width:100%; height:100%;'>"+
                        "<tr><td colspan='3' style='height:50%;'></td></tr>"+
                        "<tr><td width='50%'></td><td width='52'><img src='/img/loading.gif' width='52' height='52' alt='loading'></td><td width='50%'></td></tr>"+
                        "<tr><td colspan='3' style='height:50%;'></td></tr>"+
                        "</table>";
        this.animation.append($(loading));
        $("body").append(this.animation);
        /** number of requests to show LOADING */
        this.index = 0;
        /**private
         * request to show LOADING view
         * counts all requests
         */
        this.start = function() {
            this.index++;
            if (this.index > 0) {
                this.animation.show();
            }
        };
        /**private
         * request to hide LOADING view
         * if all requesters requested "hide", then real hide occur
         */
        this.stop = function() {
            if (this.index) {
                this.index--;
            }
            if (!this.index) {
                this.animation.hide();
            }
        };
    };
    /**
     * delete LOADING view from page
     */
    DeskLoading.prototype.clear = function() {
        this.animation.remove();
    };

    /* Projects controller */
    /**
     * Controller of all panels related to projects
     * @param desk - main DeskObject
     * @param projectbar - element to insert projects into
     */
    var DeskProjects = function(desk, projectbar) {
        /** all projects of current user */
        this.projects = {};
        /** data about all projects to build a tree view */
        this.projectsData = [];
        /** selected item in tree */
        this.current = undefined;
        /** the tree view of all projects */
        this.tree = undefined;
        /** renderers for project types */
        this.renderers = {};
        /** projects, waiting for renderers to apply */
        this.projectsQueue = {};
        /** element to insert projects into */
        this.projectbar = projectbar;
        /** main DeskObject */
        this.desk = desk;
        /** constants */
        this.nodes = DeskProjects.nodes;
        /** renderers for projects */
        this.rendererInstances = {};

        this.reuseLoadedFactories = function(newFactories) {
            var projects = this;
            $.each(newFactories, function(i, newFactory) {
                $.each(projects.renderers, function(k, projectRenderers) {
                    $.each(projectRenderers.factories, function(l, oldFactory) {
                        if (l == i) {
                            newFactories[i] = oldFactory;
                        }
                    });
                });
            });
        };
        this.initRenderer = function(projectNode, projectType, selectedFactory) {
            if (!selectedFactory) {
                selectedFactory = undefined;
            }
            var projectId = projectNode.data.data.id + '';
            var produceRenderer = function(projectId) {
                if (typeof this.rendererInstances[projectId] == "undefined") {
                    this.rendererInstances[projectId] = {};
                }
                if (typeof selectedFactory != "undefined") {
                    if (typeof this.rendererInstances[projectId][selectedFactory] != "undefined") {
                        if (typeof this.rendererInstances[projectId][selectedFactory].filesChanged == "function") {
                            this.rendererInstances[projectId][selectedFactory].filesChanged(projectNode.data.data.files);
                        }
                        return;
                    }
                }
                if (typeof selectedFactory == "undefined") {
                    for (var i in this.renderers[projectType].factories) {
                        if (this.renderers[projectType].factories.hasOwnProperty(i) && typeof(i) !== 'function') {
                            selectedFactory = i;
                            break;
                        }
                    }
                }
                var res = this.renderers[projectType].factories[selectedFactory].produce(this.desk, projectNode, projectNode.data.data.files);
                if (!res) {
                    if (console) { console.error("For projectType " + projectType + " factory " + selectedFactory + " did not return a value"); }
                } else {
                    projectNode.data.renderer = res;
                    if (typeof this.rendererInstances[projectId] == "undefined") {
                        this.rendererInstances[projectId] = {}
                    }
                    this.rendererInstances[projectId][selectedFactory] = res;
                }
            };
            if (!projectNode.data.data.files) {
                this.desk.api.files.list(projectNode.data.data.link_name, undefined, undefined, function(data) {
                    projectNode.data.data.files = data;
                    produceRenderer.call(this, projectId);
                }, this);
            } else {
                produceRenderer.call(this, projectId);
            }
        };
        this.addProjectToTree = function(parent, project) {
            if (parent.children.length == 1 && parent.children[0].type == DeskProjects.nodes.PROJECT && parent.children[0].data == "none") {
                parent.children.pop();
            }
            var projectNode = {};
            projectNode.label = project.name;
            projectNode.children = [];
            projectNode.data = {data: project};
            projectNode.type = DeskProjects.nodes.PROJECT;
            project.view = projectNode;
            parent.children.push(projectNode);
            this.startRenderer(project);
        };
    };
    /* types of project nodes */
    DeskProjects.nodes = {
        GROUP: "group",
        PROJECT: "project"
    };
    /**
     * request all projects of current user and show them on page
     */
    DeskProjects.prototype.init = function() {
        this.desk.loading.start();
        this.desk.api.projects.list(function(data) {
            this.projects = data;
            this.refresh();
            this.desk.loading.stop();
        }, this);
    };
    /**
     * delete controller
     */
    DeskProjects.prototype.clear = function() {
    };
    /**
     * Change selected node
     */
    DeskProjects.prototype.currentChanged = function(node) {
        this.current = node;
        if (node && node.type) {
            if (node.type == DeskProjects.nodes.GROUP) {
                Desk.current.context.contextChange(Desk.Common.states.PROJECT_GROUP, node);
                return;
            }
            if (node.type == DeskProjects.nodes.PROJECT) {
                if (node.data == "none") {
                    Desk.current.context.contextChange(Desk.Common.states.NONE, undefined);
                    return;
                } else {
                    Desk.current.context.contextChange(Desk.Common.states.PROJECT, node);
                    return;
                }
            }

            if (this.current.data.renderer) {
                if (this.current.data.renderer.currentChanged && this.current.data.renderer.currentChanged(this.current)) {
                    return;
                }
            }
        }
        Desk.current.context.contextChange(Desk.Common.states.NONE, undefined);
    };
    /**
     * clean the tree view and rebuild it
     */
    DeskProjects.prototype.refresh = function() {
        if (this.tree) {
            this.tree.remove();
        }
        this.projectsData = [];
        this.tree = $("<div></div>");
        this.projectbar.append(this.tree);
        for (var key in this.projects) {
            if (!this.projects.hasOwnProperty(key)) {
                continue;
            }
            var group = {};
            group.label = key;
            group.children = [];
            group.data = {data: this.projects[key]};
            group.type = DeskProjects.nodes.GROUP;
            this.projects[key].view = group;
            // collect projects in group
            if (this.projects[key] && this.projects[key].length) {
                for (var i = 0; i < this.projects[key].length; i++) {
                    this.addProjectToTree(group, this.projects[key][i]);
                }
            } else {
                group.children.push({label: "No projects added", data: "none", type: DeskProjects.nodes.PROJECT});
            }
            this.projectsData.push(group);
        }
        this.tree.tree({
            data: this.projectsData,
            autoOpen: 0,
            saveState: true,
            selectable: true
        });
        this.tree.bind(
            'tree.click',
            function() {
                var that = this;
                return function(event) {
                    that.currentChanged(event.node);
                };
            }.call(this)
        );
    };
    /**
     * reload tree structure into the tree view
     */
    DeskProjects.prototype.reloadData = function() {
        this.tree.tree('loadData', this.projectsData);
    };
    /**
     * find project in all lists
     * @param project - project to find
     */
    DeskProjects.prototype.checkProject = function(project) {
        var res = undefined;
        var that = this;
        $.each(["reader", "writer", "owner"], function(k, key) {
            $.each(that.projects[key], function(i, el) {
                if(el.id == project.id) {
                    res = {list: key, pos: i, project: el};
                }
            });
        });
        return res;
    };
    /**
     * add another project and own it
     * @param project - project to be added
     */
    DeskProjects.prototype.addProject = function(project) {
        this.projects["owner"].push(project);
        this.addProjectToTree(this.projects["owner"].view, project);
        this.reloadData();
    };
    /**
     * include another project in list
     * @param project - project to be included
     */
    DeskProjects.prototype.includeProject = function(project, rights) {
        if (!rights) {
            rights = "reader";
        }
        var t = this.checkProject(project);
        if (!t) {
            this.projects[rights].push(project);
            this.addProjectToTree(this.projects[rights].view, project);
            this.reloadData();
        } else {
            //TODO: move project to another list (if its higher in rights)
        }
    };
    /**
     * delete project from list
     * @param project - project to be deleted
     */
    DeskProjects.prototype.deleteProject = function(project, list) {
        var i = -1;
        if (!list) {
            var t = this.checkProject(project);
            if (t) {
                list = t.list;
                i = t.pos;
            }
        }
        if (!list) {
            return;
        }
        if (i < 0) {
            return;
        }
        this.projects[list].splice(i, 1);
        this.projects[list].view.children.splice(i, 1);
        if (this.projects[list].view.children.length == 0) {
            this.projects[list].view.children.push({label: "No projects added", data: "none", type: DeskProjects.nodes.PROJECT});
        }
        if (this.current.data.data == project) {
            this.currentChanged(undefined);
        }
        this.reloadData();
    };
    /**
     * load renderer for project
     */
    DeskProjects.prototype.startRenderer = function(project) {
        var typeId = project.type.id + '';
        if (typeof this.renderers[typeId] == "undefined") {
            // enqueue project
            if (typeof this.projectsQueue[typeId] == "undefined") {
                this.projectsQueue[typeId] = []
            }
            this.projectsQueue[typeId].push(project.view);
            this.renderers[typeId] = this.desk.api.projects.renderer(
                typeId,
                function(type) {
                        var that = this;
                        return function(factories, loadDescriptor) {
                                loadDescriptor.factories = factories;
                                that.reuseLoadedFactories(factories);
                                if (typeof that.projectsQueue[typeId] != "undefined") {
                                    for (var i = 0; i < that.projectsQueue[type].length; i++) {
                                        that.initRenderer(that.projectsQueue[type][i], type);
                                    }
                                }
                            };
                    }.call(this, typeId),
                this);
        } else {
            this.initRenderer(project.view, typeId);
        }
    };
    /**
     * show project properties on properties bar
     */
    DeskProjects.prototype.projectSelectState = function(source, element, menu, propertiesObj, node) {
        if (node.data.data.owner.id == this.desk.account.id) {
            var params = [
                {"name": "description", "title": "Description", "type": "text", "options": [], "value": node.data.data.description},
                {"name": "public", "title": "Is public", "type": "checkbox", "options": [], "value": node.data.data.public},
                {"name": "status", "title": "Status", "type": "select", "options": [{"value": 0, "text": "ACTIVE"}, {"text": "SHELVED", "value": 1}, {"text": "ARCHIVED", "value": 2}], "value": node.data.data.status}
            ];
            propertiesObj.showProperties(params, function(properties) {
                this.desk.api.projects.setProperties(node.data.data.link_name, properties, function() {
                    $.extend(node.data.data, properties);
                });
            }, this);
        } else {
            propertiesObj.cleanProperties();
        }
    };

    /* Files controller */
    /**
     * Controller of all panels related to files
     * @param desk - main DeskObject
     * @param editorZone - element to insert editors into
     */
    var DeskFiles = function(desk, editorZone) {
        /** all files of current user */
        this.files = {};
        /** selected item in editor */
        this.current = undefined;
        /** editors for file types */
        this.editors = {};
        /** files, waiting for editors to apply */
        this.filesQueue = {};
        /** element to insert files into */
        this.editorZone = editorZone;
        /** main DeskObject */
        this.desk = desk;
        /** editors for files */
        this.editorInstances = {};
        /** modification types */
        this.modificationTypes = DeskFiles.Modification.types;
        /** modification branched */
        this.paths = {};
        this.paths[DeskFiles.Modification.types.INSERT] = function(file, modification) {
            file.content = file.content.substring(0, modification.start) + modification.newValue + file.content.substring(modification.start);
        };
        this.paths[DeskFiles.Modification.types.REPLACE] = function(file, modification) {
            file.content = file.content.substring(0, modification.start) + modification.newValue + file.content.substring(modification.start + modification.length);
        };
        this.paths[DeskFiles.Modification.types.DELETE] = function(file, modification) {
            file.content = file.content.substring(0, modification.start) + file.content.substring(modification.start + modification.length);
        };
        /** rollback modification branched */
        this.rollbackPaths = {};
        this.rollbackPaths[DeskFiles.Modification.types.INSERT] = function(modification) {
            var res = new DeskFiles.Modification(undefined, modification.file, DeskFiles.Modification.types.DELETE, modification.start, modification.newValue.length, "");
            res.newFile = res.file;
            return res;
        };
        this.rollbackPaths[DeskFiles.Modification.types.REPLACE] = function( modification) {
            var res = new DeskFiles.Modification(undefined, modification.file, DeskFiles.Modification.types.REPLACE, modification.start, modification.newValue.length, modification.file.content.substr(modification.start, modification.length));
            res.newFile = res.file;
            return res;
        };
        this.rollbackPaths[DeskFiles.Modification.types.DELETE] = function(modification) {
            var res = new DeskFiles.Modification(undefined, modification.file, DeskFiles.Modification.types.INSERT, modification.start, 0, modification.file.content.substr(modification.start, modification.length));
            res.newFile = res.file;
            return res;
        };

        this.initEditor = function(project, file, fileType, selectedFactory, silent) {
            if (!selectedFactory) {
                selectedFactory = undefined;
            }
            var fileId = file.id + '';
            if (typeof this.editorInstances[fileId] == "undefined") {
                this.editorInstances[fileId] = {};
            }
            if (typeof selectedFactory != "undefined") {
                if (typeof this.editorInstances[fileId][selectedFactory] != "undefined") {
                    if (typeof this.editorInstances[fileId][selectedFactory].fileChanged == "function") {
                        var m = new DeskFiles.Modification(undefined, file, undefined, 0, 0, file.content);
                        m.newFile = file;
                        this.editorInstances[fileId][selectedFactory].fileChanged(m);
                    }
                    if (!silent && typeof this.editorInstances[fileId][selectedFactory].activated == "function") {
                        this.editorInstances[fileId][selectedFactory].activated(this.editorZone);
                    }
                    return;
                }
            }
            var producer = function() {
                if (typeof selectedFactory == "undefined") {
                    for (var i in this.editors[fileType].factories) {
                        if (this.editors[fileType].factories.hasOwnProperty(i) && typeof(i) !== 'function') {
                            selectedFactory = i;
                            break;
                        }
                    }
                }
                if (this.editors[fileType] && this.editors[fileType].factories[selectedFactory]) {
                    var res = this.editors[fileType].factories[selectedFactory].produce(this.desk, project, file);
                    if (!res) {
                        if (console) { console.log("For fileType " + fileType + " factory " + selectedFactory + " did not return a value"); }
                    } else {
                        if (typeof this.editorInstances[fileId] == "undefined") {
                            this.editorInstances[fileId] = {}
                        }
                        this.editorInstances[fileId][selectedFactory] = res;
                        if (!silent && typeof this.editorInstances[fileId][selectedFactory].activated == "function") {
                            this.editorInstances[fileId][selectedFactory].activated(this.editorZone);
                        }
                    }
                } else {
                    if (console) { console.error("No editor factory for " + fileType + " and " + selectedFactory); }
                }
            };
            if (typeof file.content == "undefined") {
                this.desk.api.files.get(project.link_name, file, function(data) {
                    file.content = data;
                    producer.call(this);
                }, this);
            } else {
                producer.call(this);
            }
        };
    };
    /**
     * delete controller
     */
    DeskFiles.prototype.clear = function() {
    };
    /**
     * create context for adding file
     */
    DeskFiles.prototype.addFileState = function(source, element, menu, projectNode) {
        var s = $("<div></div>");
        s.attr("title", "Add file");
        s.addClass("icon-button add-file-icon-button");
        s.click(function() {
            //add new file with owner = user
            s.attr("disabled", "disabled");
            Desk.current.loading.start();
            Desk.current.api.projectTypes.listFiles(projectNode.data.data.type.id, function(data) {
                var d = $("<div title='Add new file'></div>");
                var name = $("<input type='text'/>");
                var type = $("<select></select>");
                d.append(name);
                d.append(type);
                type.append("<option value='' disabled selected>-</option>");
                for (var i = 0; i < data.length; i++) {
                    type.append("<option value='" + data[i].id + "'>" + data[i].name + "</option>");
                }
                $("body").append(d);
                d.dialog({
                    resizable: false,
                    width: 300,
                    height:240,
                    modal: true,
                    buttons: {
                        "Add": function() {
                            if (!name.val().length) {
                                return;
                            }
                            if (!type.val()) {
                                return;
                            }
                            Desk.current.loading.start();
                            Desk.current.api.files.add(projectNode.data.data.link_name, undefined, undefined, "", name.val(), type.val(), "", function(data) {
                                    source.addFileToList(data.file);
                                    Desk.current.loading.stop();
                                    $(this).dialog("close");
                                },
                                this,
                                function(data) {
                                    Desk.current.loading.stop();
                                });
                        },
                        "Cancel": function() {
                            $(this).dialog("close");
                        }
                    }
                });
                Desk.current.loading.stop();
            });
        });
        menu.append(s);
    };
    /**
     * create context for deleting file
     */
    DeskFiles.prototype.deleteFileState = function(source, element, menu, projectNode) {
        var s = $("<div></div>");
        s.attr("title", "Delete file");
        s.addClass("icon-button delete-file-icon-button");
        s.click(function() {
            s.attr("disabled", "disabled");
            Desk.current.loading.start();
            Desk.current.api.files.remove(projectNode.data.data.link_name, undefined, undefined, element.data.data.id, function(data) {
                    source.deleteFileFromList(element.data.data);
                    Desk.current.loading.stop();
                },
                undefined,
                function(data) {
                    Desk.current.loading.stop();
                });
        });
        menu.append(s);
    };
    /**
     *
     */
    DeskFiles.prototype.showSelectFileEditor = function(source, element, menu, projectNode) {
        var s = $("<div></div>");
        s.attr("title", "Change editor");
        s.addClass("select-box");
        var editorSelect = $("<select></select>");
        $.each(this.editors[element.data.data.type.id + ''].factories, function(i, factory) {
            editorSelect.append($("<option value='" + i + "'>" + factory.editorName() + "</option>"));
        });
        s.append(editorSelect);
        editorSelect.change(this, function(event) {
            event.data.startEditor(projectNode.data.data, element.data.data, editorSelect.val());
        });
        menu.append(s);
    };
    /**
     * create modification object
     */
    DeskFiles.prototype.createModification = function(source, file, type, start, length, newValue) {
        return new DeskFiles.Modification(source, file, type, start, length, newValue);
    };
    /**
     * prepare modification object
     */
    DeskFiles.prototype.notifyModification = function(project, source, file, newValue) {
        if (typeof file.veryOldContent == "undefined") {
            file.veryOldContent = file.content;
        }
        var diff = this.diffStrings(file.content, newValue);
        console.log("changes: " + diff.length);
        var m = this.buildModification(source, file, diff);
        if (typeof m != "undefined") {
            console.log("Detected " + m.type + ": " + m.start + " - " + (m.start + m.length) + " = " + m.newValue);
            this.deferredApplyModification(project, file, m);
        }
    };
    /**
     * build modification object from diff
     */
    DeskFiles.prototype.buildModification = function(source, file, diff) {
        var diffStart;
        var textStart = 0;
        for(diffStart = 0; diffStart < diff.length && diff[diffStart][0] == 0; diffStart++) { textStart += diff[diffStart][1].length; }
        if (diffStart < diff.length) {
            var insertValue = (diff[diffStart][0] > 0 ? diff[diffStart][1] : "");
            var diffType = diff[diffStart][0];
            var diffEnd = diffStart + 1;
            var textLength = (diff[diffStart][0] < 0 ? diff[diffStart][1].length : 0);
            var passedLength = 0;
            var passedType = 0;
            var passedText = "";
            for(var l = diffEnd; l < diff.length; l++) {
                if (passedType == 0) {
                    passedType = diff[l][0];
                } else {
                    if (passedType < 2 && passedType != diff[l][0]) {
                        passedType = 2;
                    }
                }
                if (diff[l][0] <= 0) {
                    passedLength += diff[l][1].length;
                }
                if (diff[l][0] >= 0) {
                    passedText += diff[l][1];
                }
                if (diff[l][0] != 0) {
                    diffEnd = l + 1;
                    insertValue += passedText;
                    passedText = "";
                    textLength += passedLength;
                    passedLength = 0;
                    if (passedType != diffType) {
                        diffType = 2;
                    }
                    passedType = 0;
                }
            }
            if (diffType != 0) {
                var types = [this.modificationTypes.DELETE, undefined, this.modificationTypes.INSERT, this.modificationTypes.REPLACE];
                return new this.createModification(source, file, types[diffType + 1], textStart, textLength, insertValue);
            }
        }
        return undefined;
    };
    /**
     * broadcast modification object with delayed server request
     */
    DeskFiles.prototype.deferredApplyModification = function(project, file, modification) {
        modification.newFile = file;
        this.paths[modification.type](file, modification);
        for(var factory in this.editorInstances[file.id + '']) {
            this.editorInstances[file.id + ''][factory].fileChanged(modification);
        }
        if (this.deferredApplyModificationTimeout) {
            window.clearTimeout(this.deferredApplyModificationTimeout);
        }
        this.deferredApplyModificationTimeout = window.setTimeout(this.desk.utils.safeInvoke(function() {
            var thiz = this;
            return function() {
                if (typeof file.veryOldContent != "undefined") {
                    var diff = thiz.diffStrings(file.veryOldContent, file.content);
                    console.log("changes: " + diff.length);
                    var m = thiz.buildModification(thiz, file, diff);
                    if (typeof m != "undefined") {
                        thiz.applyModification(project, file, m);
                    }
                }
            };
        }, this), 5000);
    };
    /**
     * broadcast modification object
     */
    DeskFiles.prototype.applyModification = function(project, file, modification) {
        if (typeof this.editorInstances[file.id + ''] == "undefined") {
            if (console) { console.log("Missing file changed " + file.id); }
            return;
        }
        console.log(">" + file.content);
        this.desk.api.files.edit(project.link_name, undefined, undefined, file.id, modification, function(data) {
            console.log("<" + data.file.content);
            if (file.data) {
                file.data.data.data = data.file;
                data.file.data = file.data;
                file.data = undefined;
            }
            modification.newFile = data.file;
            for(var factory in this.editorInstances[file.id + '']) {
                this.editorInstances[file.id + ''][factory].fileChanged(modification);
                if (typeof this.editorInstances[modification.newFile.id + ''] == 'undefined') {
                    this.editorInstances[modification.newFile.id + ''] = {};
                }
                this.editorInstances[modification.newFile.id + ''][factory] = this.editorInstances[file.id + ''][factory];
            }
        }, this, function() {
            if (modification.source) {
                modification.source.fileChanged(this.rollbackModification(modification));
            }
        });
    };
    /**
     * create rollback of modification object
     */
    DeskFiles.prototype.rollbackModification = function(modification) {
        return this.rollbackPaths[modification.type](modification);
    };
    /**
     * signup for modification broadcast
     */
    DeskFiles.prototype.listenModification = function(file, callback, callbackObj) {
        if (typeof this.editorInstances[file.id + ''] == "undefined") {
            this.editorInstances[file.id + ''] = {};
        }
        var n = Math.random() + '';
        while(typeof this.editorInstances[file.id + ''][n] != "undefined") {
            n = Math.random() + '';
        }
        this.editorInstances[file.id + ''][n] = new DeskFiles.ModificationListener(callback, callbackObj);
    };
    /**
     * load editor for file
     */
    DeskFiles.prototype.startEditor = function(project, file, selectedFactory, silent) {
        if(!selectedFactory) {
            selectedFactory = undefined;
        }
        var typeId = file.type.id + '';
        if (typeof this.editors[typeId] == "undefined") {
            // enqueue project
            if (typeof this.filesQueue[typeId] == "undefined") {
                this.filesQueue[typeId] = []
            }
            this.filesQueue[typeId].push(file);
            this.editors[typeId] = this.desk.api.files.editor(
                typeId,
                function(type) {
                        var that = this;
                        return function(factories, loadDescriptor) {
                                loadDescriptor.factories = factories;
                                if (typeof that.filesQueue[typeId] != "undefined") {
                                    for (var i = 0; i < that.filesQueue[type].length; i++) {
                                        that.initEditor(project, that.filesQueue[type][i], type, selectedFactory, silent);
                                    }
                                }
                            };
                    }.call(this, typeId),
                this);
        } else {
            this.initEditor(project, file, typeId, selectedFactory, silent);
        }
    };
    /**
     * find differences between strings
     */
    DeskFiles.prototype.diffStrings = function(oldString, newString) {
        var dmp = new diff_match_patch();
        var diff = dmp.diff_main(oldString, newString);
        dmp.diff_cleanupEfficiency(diff);
        return diff;
    };
    /**
     * Dummy Editor for listener signup
     */
    DeskFiles.ModificationListener = function(callback, callbackObj) {
        this.callback = callback;
        this.callbackObj = callbackObj;
    };
    DeskFiles.ModificationListener.prototype.fileChanged = function(modification) {
        this.callback.call(this.callbackObj, modification);
    };
    /**
     * File modification descriptor
     */
    DeskFiles.Modification = function(source, file, type, start, length, newValue) {
        this.source = source;
        this.file = file;
        this.newFile = undefined;
        this.type = type;
        this.start = start;
        this.length = length;
        this.newValue = newValue;
    };
    DeskFiles.Modification.types = {
        INSERT: "insert",
        REPLACE: "replace",
        DELETE: "delete"
    };

    /* Whole context of the IDE */
    /**
     * Controller of IDE context
     * @param desk - main DeskObject
     * @param toolbar - element to insert context menu into
     */
    var DeskContext = function(desk, toolbar, propertiesObj) {
        /** registered context states and actions on them */
        this.contextActions = {};
        /** current context state */
        this.currentContext = "";
        /** main DeskObject */
        this.desk = desk;
        /** element to insert context menu into */
        this.toolbar = toolbar;
        /** properties controller */
        this.propertiesObj = propertiesObj;
    };
    /**
     * register reaction for a context state
     * @param context - context state
     * @param func - reaction callback
     * @param thisObj - reaction callback "this"
     */
    DeskContext.prototype.registerContext = function(context, func, thisObj) {
        if (!this.contextActions[context]) {
            this.contextActions[context] = [];
        }
        this.contextActions[context].push({"function":func,"this":thisObj});
    };
    /**
     * change context state and execute reactions
     * @param context - new context state
     * @param element - source of change (any object)
     */
    DeskContext.prototype.contextChange = function(context, element) {
        this.toolbar.children().remove();
        if (this.contextActions[Desk.Common.states.ANY]) {
            $.each(this.contextActions[Desk.Common.states.ANY], function(that) {
                return function(i, func) {
                    func["function"].call(func["this"], element, that.toolbar, that.propertiesObj);
                };
            }(this));
        }
        if (this.contextActions[context]) {
            $.each(this.contextActions[context], function(that) {
                return function(i, func) {
                    func["function"].call(func["this"], element, that.toolbar, that.propertiesObj);
                }
            }(this));
        }
    };

    /* Properties controller *///TODO:
    var DeskProperties = function(desk, propertiesBar) {
        /** main DeskObject */
        this.desk = desk;
        /** element to insert properties into */
        this.propertiesBar = propertiesBar;
    };
    /**
     * remove properties from properties pane
     */
    DeskProperties.prototype.cleanProperties = function() {
        this.propertiesBar.children().remove();
    };
    /**
     * show properties on properties pane
     * @param properties - array of {"name":..., "title":..., "type":..., "options": [], "value":...}
     * @param func - callback on property change
     * @param thisObj - callback "this"
     */
    DeskProperties.prototype.showProperties = function(properties, func, thisObj) {
        var controller = this;
        this.propertiesBar.children().remove();
        var collect = [];
        $.each(properties, function(i, property) {
            var item = undefined;
            if (property.type == "select") {
                item = $("<select></select>");
                $.each(property.options, function(k, o) {
                    item.append("<option value='" + o.value + "'>" + o.text + "</option>");
                });
            }
            if (property.type == "checkbox") {
                item = $("<input type='checkbox'>");
            }
            if (typeof item == "undefined") {
                item = $("<input type='text'>");
            }
            item.attr('name', property.name);
            if (typeof property.value != "undefined") {
                item.val(property.value);
                if (property.type == "checkbox") {
                    item.prop('checked', new Boolean(property.value));
                }
            }
            item.attr('placeholder', property.title);
            item.change(function() {
                var res = {};
                $.each(collect, function(i, el) {
                    if (el.is("input[type=checkbox]")) {
                        res[el.attr('name')] = el.is(":checked");
                    } else {
                        res[el.attr('name')] = el.val();
                    }
                });
                controller.desk.utils.safeInvoke(func, thisObj, [res]);
            });
            collect.push(item);
            controller.propertiesBar.append(item);
            if (property.type == "checkbox") {
                controller.propertiesBar.append("<span>&nbsp;" + property.title + "</span>");
            }
            controller.propertiesBar.append("<br>");
        });
    };

    /* Status controller */

    /* Utils */
    var DeskUtils = function() {
    };
    DeskUtils.prototype.generateRandom = function(length) {
        var chars = '0123456789abcdefghiklmnopqrstuvwxyz'.split('');
        if (!length) {
            length = Math.floor(Math.random() * chars.length);
        }
        var str = '';
        for (var i = 0; i < length; i++) {
            str += chars[Math.floor(Math.random() * chars.length)];
        }
        return str;
    };
    DeskUtils.prototype.trim = function(str) {
        str = str.replace(/^\s+/, '');
        for (var i = str.length - 1; i >= 0; i--) {
            if (/\S/.test(str.charAt(i))) {
                str = str.substring(0, i + 1);
                break;
            }
        }
        return str;
    };
    DeskUtils.prototype.safeInvoke = function(func, scope, params) {
        if(typeof func != "function") {
            func = function(){};
        }
        if(typeof params == "undefined" || typeof params.length == "undefined") {
            params = [params];
        }
        return func.apply(scope, params);
    };

    /* Main Desk Object*/
    /**
     * Controller of all controllers and the structure of IDE
     */
    var DeskObject = function() {
        /** server address */
        var temp = document.location.href.substring(document.location.href.indexOf('://') + 3);
        this.server = document.location.href.substring(0, document.location.href.indexOf('://') + 3) + temp.substring(0, temp.indexOf('/'));
        /** user's account id */
        this.account = undefined;
        //create layout
        /** element to insert work desk into */
        this.container = jqElement;
        /** logo zone */
        this.logo = $('<div class="ui-layout-north" style="text-align: center;"><img src="/img/flightprocess.png" height="100" alt="FlightProcess" title="FlightProcess" /></div>');
        /** main zone - work zone */
        this.workZone = $('<div class="outer-center"></div>');
        /** status bar zone */
        this.statusBar = $('<div class="ui-layout-south"></div>);');
        /** zone for a list of projects */
        this.projectList = $('<div class="middle-west"></div>');
        /** zone for any properties */
        this.propertiesBar = $('<div class="middle-east"></div>');
        /** zone for editor */
        this.editor = $('<div class="middle-center"></div>');
        /** zone for context menu */
        this.toolBar = $('<div class="ui-layout-north"></div>');
        this.toolBar.addClass("toolbar");
        this.container.append(this.workZone);
        this.container.append(this.logo);
        this.container.append(this.statusBar);
        this.workZone.append(this.editor);
        this.workZone.append(this.projectList);
        this.workZone.append(this.propertiesBar);
        this.workZone.append(this.toolBar);
        /** common LOADING view */
        this.loading = new DeskLoading();
        /** utils */
        this.utils = new DeskUtils();

        var outerLayout = this.container.layout({
                    center__paneSelector:	".outer-center"
                ,	spacing_open:			8
                ,	spacing_closed:			12
                ,	north__minSize:			100
                ,	north__maxSize:			100
                ,	south__maxSize:			200
            });

        var middleLayout = this.container.find('div.outer-center').layout({
                    center__paneSelector:	".middle-center"
                ,   center__onresize: function() {
                    }
                ,	west__paneSelector:		".middle-west"
                ,	east__paneSelector:		".middle-east"
                ,	north__size:			50
                ,	north__minSize:			50
                ,	north__maxSize:			50
                ,	west__size:				200
                ,	west__minSize:			50
                ,	west__maxSize:			400
                ,	east__size:				200
                ,	east__minSize:			50
                ,	east__maxSize:			400
                ,	spacing_open:			8
                ,	spacing_closed:			12
            });

        /**private
         * reset scroll bars on projects, any properties and status bar
         */
        this.scrolledItems = [];
        this.setScrolls = function() {
            var settings = {
                showArrows: false,
                autoReinitialise: true
            };
            var thiz = this;
            $.each(['projectList', 'propertiesBar', 'statusBar'], function(i, el) {
                thiz[el].jScrollPane(settings);
                thiz.scrolledItems.push(thiz[el]);
                thiz[el] = thiz[el].data('jsp').getContentPane();
            });
        };
        this.resetScrolls = function() {
            $.each(this.scrolledItems, function(i, el) {
                el.data('jsp').reinitialise();
            });
        };
        /* on any resize, scrolls need to be reset */
        this.setScrolls();
        this.scrollBarTimeout = undefined;
        $(window).bind(
            'resize',
            this.utils.safeInvoke(function()
            {
                var thiz = this;
                return function() {
                    if ($.browser.msie) {
                        // IE fires multiple resize events while you are dragging the browser window which
                        // causes it to crash if you try to update the scrollpane on every one. So we need
                        // to throttle it to fire a maximum of once every 50 milliseconds...
                        if (!thiz.scrollBarTimeout) {
                            thiz.scrollBarTimeout = setTimeout(
                                function()
                                {
                                    thiz.resetScrolls();
                                    thiz.scrollBarTimeout = null;
                                },
                                50
                            );
                        }
                    } else {
                        thiz.resetScrolls();
                    }
                }
            }, this)
        );

        /** projects controller */
        this.projectObj = new DeskProjects(this, this.projectList);

        /** files controller */
        this.fileObj = new DeskFiles(this, this.editor);

        /** properties controller */
        this.propertiesObj = new DeskProperties(this, this.propertiesBar);

        /** IDE server API */
        this.api = DeskApi;

        /** context of IDE */
        this.context = new DeskContext(this, this.toolBar, this.propertiesObj);

        this.api.users.current(function(data) {
            this.account = data;
        }, this);
        this.projectObj.init();
    };
    /**
     * remove all elements of work desk
     */
    DeskObject.prototype.clear = function() {
        this.container.children().remove();
        this.loading.clear();
        this.projectObj.clear();
    };

    /* singleton pattern */
    if (Desk.current) {
        Desk.current.clear();
    }
    Desk.current = new DeskObject();

    for(var i = 0; i < Desk.registerQueue.length; i++) {
        Desk.current.context.registerContext(Desk.registerQueue[i].context, Desk.registerQueue[i].func, Desk.registerQueue[i].thisObj);
    }

    /* initialization context - state "none" */
    Desk.current.context.contextChange(Desk.Common.states.NONE, undefined);
};

/** queue of context states if desk is not initialized */
Desk.registerQueue = [];
/**
 * static interface for context registration
 * @param context - context state
 * @param func - reaction callback
 * @param thisObj - reaction callback "this"
 */
Desk.registerContext = function(context, func, thisObj) {
    if (Desk.current) {
        if (Desk.current.context) {
            Desk.current.context.registerContext(context, func, thisObj);
        } else {
            throw new Error("Desk context is corrupted", 11);
        }
    } else {
        Desk.registerQueue.push({context: context, func: func, thisObj: thisObj});
        //throw new Error("Desk is not initialized", 10);
    }
};

/* Common State */
Desk.Common = {};
Desk.Common.states = {
    ANY: "desk.any",
    NONE: "desk.none",
    PROJECT_GROUP: "desk.projectGroup",
    PROJECT: "desk.project"
};

/* context for adding project */
Desk.Common.addProject = function(element, menu) {
    var s = $("<div></div>");
    s.attr("title", "Add project");
    s.addClass("icon-button add-icon-button");
    s.click(function() {
        //add new project with owner = user
        s.attr("disabled", "disabled");
        Desk.current.loading.start();
        Desk.current.api.projectTypes.list(function(data) {
            var d = $("<div title='Add new project'></div>");
            var name = $("<input type='text'/>");
            var type = $("<select></select>");
            d.append(name);
            d.append(type);
            type.append("<option value='' disabled selected>-</option>");
            for (var i = 0; i < data.length; i++) {
                type.append("<option value='" + data[i].id + "'>" + data[i].name + "</option>");
            }
            $("body").append(d);
            d.dialog({
                resizable: false,
                width: 300,
                height:240,
                modal: true,
                buttons: {
                    "Add": function() {
                        if (!name.val().length) {
                            return;
                        }
                        if (!type.val()) {
                            return;
                        }
                        Desk.current.loading.start();
                        Desk.current.api.projects.add(name.val(), type.val(), function(data) {
                                Desk.current.projectObj.addProject(data);
                                Desk.current.loading.stop();
                                $(this).dialog("close");
                            },
                            this,
                            function(data) {
                                Desk.current.loading.stop();
                            });
                    },
                    "Cancel": function() {
                        $(this).dialog("close");
                    }
                }
            });
            Desk.current.loading.stop();
        });
    });
    menu.append(s);
};

/* context for searching project */
Desk.Common.findProject = function(element, menu) {
    var s = $("<div></div>");
    s.attr("title", "Find project");
    s.addClass("icon-button find-icon-button");
    s.click(function() {
        //find project - open picker with search for public projects for read participation
        s.attr("disabled", "disabled");
        Desk.current.loading.start();
        Desk.current.api.projectTypes.list(function(data) {
            var d = $("<div title='Find project'></div>");
            var name = $("<input type='text'/>");
            var type = $("<select></select>");
            var results = $("<div></div>");
            d.append(name);
            d.append(type);
            d.append(results);
            type.append("<option value='' disabled selected>-</option>");
            for (var i = 0; i < data.length; i++) {
                type.append("<option value='" + data[i].id + "'>" + data[i].name + "</option>");
            }
            $("body").append(d);
            d.dialog({
                resizable: false,
                width: 500,
                height:440,
                modal: true,
                buttons: {
                    "Find": function() {
                        if (!name.val().length) {
                            return;
                        }
                        if (!type.val()) {
                            return;
                        }
                        results.slideUp();
                        Desk.current.loading.start();
                        Desk.current.api.projects.find(name.val(), type.val(), function(data) {
                                var dialog = $(this);
                                results.children().remove();
                                if (data.length > 0) {
                                    $.each(data, function(i, project) {
                                        var d = $("<div></div>");
                                        d.css('width', '100%');
                                        d.css('overflow', 'hidden');
                                        d.css("backgroundColor", "#ffffff");
                                        d.css("pointer", "hand");
                                        d.click(function() {
                                            Desk.current.projectObj.includeProject(project);
                                            dialog.dialog("close");
                                        });
                                        d.hover(function() {
                                            $(this).css("backgroundColor", "#aaaaaa");
                                        },function() {
                                            $(this).css("backgroundColor", "#ffffff");
                                        });
                                        d.append($("<div style='float:left;overflow:hidden;'>" + project.type.name + "</div>"));
                                        d.append($("<div style='float:right;overflow:hidden;'>" + project.name + "</div>"));
                                        d.append($("<div style='float:left;width:100%;text-align:right;'>" + (project.owner.name ? "<a href='javascript:;'"> + project.owner.name + "</a>" : "System") + "</div>"));
                                        results.append(d);
                                    });
                                } else {
                                    results.html("No results");
                                }
                                results.slideDown('fast');
                                // show found projects
                                Desk.current.loading.stop();
                            },
                            this,
                            function(data) {
                                Desk.current.loading.stop();
                            });
                    },
                    "Cancel": function() {
                        $(this).dialog("close");
                    }
                }
            });
            Desk.current.loading.stop();
        });
    });
    menu.append(s);
};

/* context for project deletion */
Desk.Common.deleteProject = function(element, menu) {
    //remove selected project
    var s = $("<div></div>");
    s.attr("title", "Delete project");
    s.addClass("icon-button delete-icon-button");
    s.click(function() {
        s.attr("disabled", "disabled");
        Desk.current.loading.start();
        (element.data.data.owner.id == Desk.current.account.id ? Desk.current.api.projects.remove : Desk.current.api.projects.exclude)(element.data.data.link_name, function(data) {
                Desk.current.projectObj.deleteProject(element.data.data);
                Desk.current.loading.stop();
            },
            undefined,
            function(data) {
                    Desk.current.loading.stop();
            });
    });
    menu.append(s);
};

/* context for file upload */
Desk.Common.uploadFile = function(element, menu) {
    //upload file to selected project
    var s = $("<div></div>");
    s.attr("title", "Upload file");
    s.addClass("icon-button upload-icon-button");
    s.click(function() {
        s.attr("disabled", "disabled");
        Desk.current.loading.start();
        var d = $("<div title='Upload file'></div>");
        var file = $("<ul>Upload file</ul>");
        d.append(file);
        $("body").append(d);
        d.dialog({
            resizable: false,
            width: 500,
            height:440,
            modal: true,
            buttons: {
                "Cancel": function() {
                    d.dialog("close");
                }
            }
        });

        var params = {
            element: file[0],
            action: "/api/files/upload/" + element.data.data.link_name,
            forceMultipart: true,
            demoMode: false,
            inputName: 'file',
            debug: true,
            hideDropzones: true,
            disableDefaultDropzone: false,
            failedUploadTextDisplay: {
                mode: 'custom',
                maxChars: 40,
                responseProperty: 'error',
                enableTooltip: true
            },
            onComplete: function(id, fileName, responseJSON) {
                $.each(responseJSON.data.files, function(i, file) {
                    element.data.renderer.addFileToList(file);
                });
                d.dialog("close");
            }
        };
        if (file.html().length > 0) {
            params['uploadButtonText'] = file.html();
        }
        new qq.FileUploader(params);

        Desk.current.loading.stop();
    });
    menu.append(s);
};

/* registration of basic contexts */
Desk.registerContext(Desk.Common.states.PROJECT, Desk.Common.deleteProject, undefined);
Desk.registerContext(Desk.Common.states.ANY, Desk.Common.addProject, undefined);
Desk.registerContext(Desk.Common.states.ANY, Desk.Common.findProject, undefined);
Desk.registerContext(Desk.Common.states.PROJECT, Desk.Common.uploadFile, undefined);
