# coding=UTF-8
import os
from django.http import HttpResponse, Http404
from django.shortcuts import redirect
from django.utils import simplejson
from google.appengine.ext import db
from google.appengine.ext.webapp import template
from web.views import ActualController
from web.services import account, project, files
from web import models
from web.util import PathUtil, CommonUtil
from process import settings

__author__ = 'IoS'

class FileController(ActualController):
    def list(self, request, url, user, method, project_name, *parts):
        state_id = request.GET.get("state_id", None)
        branch = request.GET.get("branch", None)
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if state_id is None:
            if branch is None:
                return HttpResponse(simplejson.dumps({"success": True,
                    "data": [files.FileService().apply_file_rights(files.FileService().clean_file(obj.to_dict()), a, p) for obj in files.FileService().get_project_files(p, a)]}))
            else:
                return HttpResponse(simplejson.dumps({"success": True,
                    "data": [files.FileService().apply_file_rights(files.FileService().clean_file(obj.to_dict()), a, p) for obj in files.FileService().get_last_project_state_files(p, branch, a)]}))
        else:
            state = models.ProjectState.get_by_id(int(state_id))
            if branch is not None and state.tag != branch:
                raise Http404
            return HttpResponse(simplejson.dumps({"success": True,
                "data": [files.FileService().apply_file_rights(files.FileService().clean_file(obj.to_dict()), a, p) for obj in files.FileService().get_project_state_files(state, a)]}))

    def find(self, request, url, user, project_name, *parts):
        a = account.AccountService().get_account(user)
        pr = project.ProjectService().get_project(project_name)
        if request.GET.get("name", None) is None:
            p = []
        else:
            p = files.FileService().find_files_in_project(a, request.GET.get("name", None),
                                                          pr, request.GET.get("type", None))
        return HttpResponse(simplejson.dumps({"success": True,
            "data": [files.FileService().apply_file_rights(files.FileService().clean_file(obj.to_dict()), a, pr) for obj in p]}))

    def upload(self, request, url, user, method, project_name, *parts):
        state_id = request.GET.get("state_id", None)
        branch = request.GET.get("branch", None)
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_write_rights(a, p):
            if state_id is not None:
                state = models.ProjectState.select_related([models.ProjectState.get_by_id(int(state_id))])[0]
                if state.project != p or (branch is not None and state.tag != branch):
                    raise Http404
            else:
                state = files.FileService().get_last_project_state(p, branch if branch is not None else "master")

            res = []

            for f in request.FILES.getlist('file'):
                filename = os.path.basename(f.name)
                (name, ext) = os.path.splitext(filename)
                ext = ext[1:]
                category = ""
                content = f.read()
                type = models.FileType.all().filter('extension =', ext).get()
                if type is None:
                    type = models.FileType(name = "Unknown",
                        description = "Unknown",
                        extension = ext,
                        mimetype = "text/plain")
                    type.put()

                state = files.FileService().move_state(state, p, branch if branch is not None else "master", "Add file " + category + "/" + name)
                fv = models.FileVersion(name = name,
                    category = category,
                    type = type,
                    owner = a,
                    content = db.Blob(content),
                    size = len(content))
                fv.put()
                files.FileService().add_file_to_state(state, fv)
                res.append(fv)

            return HttpResponse(simplejson.dumps({'success': True, 'data': {"state": state.to_dict(), "files": [files.FileService().apply_file_rights(files.FileService().clean_file(fv.to_dict()), a, p) for fv in res]}}))

        return HttpResponse(simplejson.dumps({"success": False}))

    def add(self, request, url, user, method, project_name, *parts):
        state_id = request.GET.get("state_id", None)
        branch = request.GET.get("branch", None)
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_write_rights(a, p):
            if state_id is not None:
                state = models.ProjectState.select_related([models.ProjectState.get_by_id(int(state_id))])[0]
                if state.project != p or (branch is not None and state.tag != branch):
                    raise Http404
            else:
                state = files.FileService().get_last_project_state(p, branch if branch is not None else "master")

            name = request.GET["name"]
            category = request.GET["category"]
            type = int(request.GET["type"])
            content = str(request.POST.get("content", str()))
            state = files.FileService().move_state(state, p, branch if branch is not None else "master",
                                            "Add file " + category + "/" + name)
            fv = models.FileVersion(name = name,
                                    category = category,
                                    type = models.FileType.get_by_id(type),
                                    owner = a,
                                    content = db.Blob(content),
                                    size = len(content))
            fv.put()
            files.FileService().add_file_to_state(state, fv)
            return HttpResponse(simplejson.dumps({"success": True, "data": {"state": state.to_dict(), "file": files.FileService().apply_file_rights(fv.to_dict(), a, p)}}))
        return HttpResponse(simplejson.dumps({"success": False}))

    def delete(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_write_rights(a, p):
            state_id = request.GET.get("state_id", None)
            branch = request.GET.get("branch", None)
            file_id = request.GET["file_id"]
            if state_id is not None:
                state = models.ProjectState.select_related([models.ProjectState.get_by_id(int(state_id))])[0]
                if state.project != p or (branch is not None and state.tag != branch):
                    raise Http404
            else:
                state = files.FileService().get_last_project_state(p, branch if branch is not None else "master")
            file = models.FileVersion.get_by_id(int(file_id))
            if file is None:
                raise Http404
            ok = False
            for fv in files.FileService().get_project_state_files(state, a):
                if fv.key().id() == file.key().id():
                    ok = True
            if ok:
                state = files.FileService().move_state(state, p, branch if branch is not None else "master", "Delete file " + file.category + "/" + file.name)
                files.FileService().delete_file_from_state(state, file)
                return HttpResponse(simplejson.dumps({"success": True, "data": state.to_dict()}))
        return HttpResponse(simplejson.dumps({"success": False}))

    def edit(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_write_rights(a, p):
            modification = simplejson.loads(request.POST["modification"])
            state_id = request.GET.get("state_id", None)
            branch = request.GET.get("branch", None)
            file_id = request.GET["file_id"]
            if state_id is not None:
                state = models.ProjectState([models.ProjectState.get_by_id(int(state_id))])[0]
                if state.project != p or (branch is not None and state.tag != branch):
                    raise Http404
            else:
                state = files.FileService().get_last_project_state(p, branch if branch is not None else "master")
            file = models.FileVersion.get_by_id(int(file_id))
            if file is None:
                raise Http404
            newcontent = files.FileService().apply_modification(file, modification)
            newfile = models.FileVersion(content = newcontent,
                                         size = len(newcontent),
                                         owner = a,
                                         name = file.name,
                                         category = file.category,
                                         type = file.type)
            newfile.put()
            state = files.FileService().move_state(state, p, branch if branch is not None else "master", "Edit file " + file.category + "/" + file.name)
            files.FileService().edit_file_to_state(state, file, newfile)
            return HttpResponse(simplejson.dumps({"success": True, "data": {"state": state.to_dict(), "file": files.FileService().apply_file_rights(newfile.to_dict(), a, p)}}))
        return HttpResponse(simplejson.dumps({"success": False}))

    def browse(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if not project.ProjectService().check_read_rights(a, p):
            return HttpResponse(template.render(PathUtil.template_path('forbidden.html'),
                                                {"project": p, "path": "/".join(parts)}),
                                status = 403)
        tag = request.GET.get("t", None)
        path = ""
        if len(parts):
            path = "/".join(parts)
            name = parts[-1]
            category = "/".join(parts[:-1])
            try:
                fv = files.FileService().try_get_file(p, tag, category, name) if tag is not None else files.FileService().try_get_master_file(p, category, name)
                return HttpResponse(fv.content, mimetype=fv.type.mimetype if fv.type.mimetype is not None else "text/plain")
            except Http404:
                if request.path.endswith('/'):
                    return template.render(PathUtil.template_path('folder.html'),
                                           {
                                               "project": p,
                                               "path": path,
                                               "nodes": files.FileService().get_level(files.FileService().try_get_path(p, tag, path) if tag is not None else files.FileService().try_get_master_path(p, path))
                                           })
                else:
                    return redirect(request.path + '/')
        if request.path.endswith('/'):
            return HttpResponse(template.render(PathUtil.template_path('folder.html'),
                               {
                                   "project": p,
                                   "path": path,
                                   "nodes": files.FileService().get_level(path, files.FileService().try_get_path(p, tag, path) if tag is not None else files.FileService().try_get_master_path(p, path))
                               }))
        else:
            return redirect(request.path + '/')

    def browse_branch(self, request, url, user, method, project_name, *parts):
        #TODO: work with branches, states and only then categories and files
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if not project.ProjectService().check_read_rights(a, p):
            return HttpResponse(template.render(PathUtil.template_path('forbidden.html'),
                    {"project": p, "path": "/".join(parts)}),
                status = 403)
        tag = request.GET.get("t", None)
        path = ""
        if len(parts):
            path = "/".join(parts)
            name = parts[-1]
            category = "/".join(parts[:-1])
            try:
                fv = files.FileService().try_get_file(p, tag, category, name) if tag is not None else files.FileService().try_get_master_file(p, category, name)
                return HttpResponse(fv.content, mimetype=fv.type.mimetype if fv.type.mimetype is not None else "text/plain")
            except Http404:
                return template.render(PathUtil.template_path('folder.html'),
                        {
                        "project": p,
                        "path": path,
                        "nodes": files.FileService().get_level(path, files.FileService().try_get_path(p, tag, path) if tag is not None else files.FileService().try_get_master_path(p, path))
                    })
        return HttpResponse(template.render(PathUtil.template_path('folder.html'),
                {
                "project": p,
                "path": path,
                "nodes": files.FileService().get_level(path, files.FileService().try_get_path(p, tag, path) if tag is not None else files.FileService().try_get_master_path(p, path))
            }))

    def editor(self, request, url, user, method, type, callback, *parts):
        callback = CommonUtil.filter_nonname_chars(callback)
        if not len(callback):
            raise Exception("Wrong callback")
        a = account.AccountService().get_account(user)
        tress = []
        fs = files.FileService()
        ps = project.ProjectService()
        for p in models.Project.all().filter('editor_available_for =', models.FileType.get_by_id(int(type))):
            if (ps.check_read_rights(a, p) and models.ApprovedEditor.all().filter('project =', p).count() > 0) or ps.check_write_rights(a, p):
                self.base_log.info("Output " + p.name)
                fc = ""
                fi = ""
                for f in fs.get_last_master_project_state_files(p):
                    if f.name + ("." + f.type.extension if len(f.type.extension) else '') == p.main_editor_source:
                        try:
                            fc = f.content.decode("utf-8")
                        except Exception:
                            fc = f.content.__str__()
                        fi = f.key().id().__str__()
                str = "\n"
                str += "function " + callback + "_run_" + p.key().id().__str__() + "() {\n"
                str += "try {\n"
                str += "function getEditorFactory(){throw new Error(\"getEditorFactory not implemented\")};\n"
                str += "var getEditorFactory = getEditorFactory;\n"
                str += fc + "\n"
                str += ";\n"
                str += "var editorFactory = getEditorFactory();\n"
                str += callback + "_arr[(typeof editorFactory[\"id\"] != \"undefined\" ? editorFactory[\"id\"] : \"" + fi + "\")] = editorFactory;\n"
                str += "}catch(e){\n"
                str += "if(console && typeof console.log == \"function\"){console.log(e);}\n"
                str += "}\n"
                str += "}\n"
                str += callback + "_run_" + p.key().id().__str__() + "();\n"
                tress.append(str)
        res = "\n"
        res += "var " + callback + "_arr = {};\n"
        res += "\n".join(tress) + "\n"
        res += callback + "(" + callback + "_arr);\n"
        return HttpResponse(res, content_type='text/javascript')

    def restore(self, request, url, user, method, *parts):
        t = open(settings.ROOT_PATH + "/axis4/neural_ide/preload/axis4-project-renderer.js", "r")
        script = t.read()
        t.close()
        t = open(settings.ROOT_PATH + "/axis4/neural_ide/preload/axis4-project-renderer-styles.css", "r")
        style = t.read()
        t.close()
        for pfv in models.FileVersion.all().filter('name =', "axis4-project-renderer"):
            if pfv.type.extension == 'css':
                pfv.content = style
            else:
                pfv.content = script
            pfv.put()

        t = open(settings.ROOT_PATH + "/axis4/neural_ide/preload/axis4-text-editor.js", "r")
        script = t.read()
        t.close()
        for pfv in models.FileVersion.all().filter('name =', "axis4-text-editor"):
            pfv.content = script
            pfv.put()

        return HttpResponse(simplejson.dumps({"success": True}))
