# coding=UTF-8
from django.http import HttpResponse
from django.utils import simplejson
from web import models
from web.views import ActualController
from web.services import account, project, files
from web.util import CommonUtil

__author__ = 'IoS'

class ProjectController(ActualController):
    def list(self, request, url, user, method, *parts):
        a = account.AccountService().get_account(user)
        return HttpResponse(simplejson.dumps({"success": True, "data": {
                'owner':[obj.to_dict() for obj in project.ProjectService().get_owned_projects(a)],
                'writer':[obj.to_dict() for obj in project.ProjectService().get_writeable_projects(a)],
                'reader':[obj.to_dict() for obj in project.ProjectService().get_readable_projects(a)],
            }}))

    def find(self, request, url, user, method, *parts):
        p = project.ProjectService().find_projects(request.GET.get("name", None), request.GET.get("type", None))
        return HttpResponse(simplejson.dumps({"success": True, "data": [obj.to_dict() for obj in p]}))

    def add(self, request, url, user, method, *parts):
        a = account.AccountService().get_account(user)
        p = models.Project(name = request.GET.get("name"), type = models.ProjectType.get_by_id(int(request.GET.get("type"))), public = bool("False"), owner = a).put()
        return HttpResponse(simplejson.dumps({"success": True, "data": models.Project.select_related([models.Project.get(p)])[0].to_dict()}))

    def delete(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_owner_rights(a, p):
            p.status = models.Project.SHELVED
            p.put()
            return HttpResponse(simplejson.dumps({"success": True}))
        return HttpResponse(simplejson.dumps({"success": False}))

    def exclude(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        for par in models.Participant.all().filter("who =", a).filter("project =", p):
            par.delete()
        return HttpResponse(simplejson.dumps({"success": True}))

    def set_properties(self, request, url, user, method, project_name, *parts):
        a = account.AccountService().get_account(user)
        p = project.ProjectService().get_project(project_name)
        if project.ProjectService().check_owner_rights(a, p):
            p.description = request.GET.get("description", p.description)
            p.public = bool(request.GET.get("public", p.public))
            p.status = int(request.GET.get("status", p.status))
            p.put()
            return HttpResponse(simplejson.dumps({"success": True}))
        return HttpResponse(simplejson.dumps({"success": False}))

    def renderer(self, request, url, user, method, type, callback, *parts):
        callback = CommonUtil.filter_nonname_chars(callback)
        if not len(callback):
            raise Exception("Wrong callback")
        a = account.AccountService().get_account(user)
        tress = []
        fs = files.FileService()
        ps = project.ProjectService()
        for p in models.Project.all().filter('renderer_available_for =', models.ProjectType.get_by_id(int(type))):
            if (ps.check_read_rights(a, p) and models.ApprovedRenderer.all().filter('project =', p).count() > 0) or ps.check_write_rights(a, p):
                fc = ""
                fi = None
                for f in fs.get_last_master_project_state_files(p):
                    if f.name + ("." + f.type.extension if len(f.type.extension) else '') == p.main_renderer_source:
                        fc = f.content
                        fi = f.key().id()
                tress.append(u"""
function %s_run_%s() {
try {
function getRendererFactory(){throw new Error("getRendererFactory not implemented")};
var getRendererFactory = getRendererFactory;
%s
;
var rendererFactory = getRendererFactory();
%s_arr[(typeof rendererFactory["id"] != "undefined" ? rendererFactory["id"] : "%s")] = rendererFactory;
}catch(e){
if(console && typeof console.log == "function"){console.log(e);}
}
}
%s_run_%s();
""" % (callback, p.key().id(), fc, callback, fi, callback, p.key().id()))
        return HttpResponse(u"""
var %s_arr = {};
%s
%s(%s_arr);
""" % (callback, "\n".join(tress), callback, callback), content_type='text/javascript')