/**
 * User: progsdi
 * Date: 12.08.12
 * Time: 01:13
 */

var FlightProcessEditor = function(desk, project, file) {
    this.desk = desk;
    this.project = project;
    this.el = undefined;
    this.jit = undefined;
    this.internalNodes = undefined;
    this.xmlNodes = undefined;
    this.visualNodes = undefined;
    this.file = file;
    this.addNode = undefined;
    this.startCompile = undefined;
    this.edgeLabelColor = '#FFA4FF';
    this.nodeRadius = 28;
    this.nodeActiveRadius = 36;
    this.preloadedEdgeImage = new Image();
    this.preloadedEdgeImage.src = Desk.current.server + '/api/files/browse/flightprocess_editor/edit-edge-icon.png';

    this.compileProcess = function(callback, callbackObj, errorCallback) {
        this.desk.api.get('/api/compile/start/' + this.project.link_name + '/', {'file_id': this.file.id, 'start_params': "{'created_by': '" + this.desk.account.user.name + "'}"}, callback, callbackObj, errorCallback);
    };
    this.getProcessFilesInLibraries = function(callback, callbackObj, errorCallback) {
        this.desk.api.get('/api/compile/processes/', undefined, callback, callbackObj, errorCallback);
    };
    this.availableProcessFiles = [];

    this.nodeTypes = {
        "takeoff": {
            title: "Plane take off",
            type: "circle",
            color: '#AAFFAA',
            params: {}
        },
        "land": {
            title: "Plane landing",
            type: "circle",
            color: '#AAAAFF',
            params: {}
        },
        "crash": {
            title: "Plane crashing",
            type: "circle",
            color: '#FFAAAA',
            params: {}
        },
        "b_unload": {
            title: "Baggage unloading service",
            type: "square",
            color: '#AAAAFF',
            params: {}
        },
        "b_load": {
            title: "Baggage loading service",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "log": {
            title: "Logging system state to remote service",
            type: "star",
            color: '#AAFFAA',
            params: {'state': 'text', 'message': 'text'}
        },
        "registration_request": {
            title: "External registration service",
            type: "star",
            color: '#FFAAAA',
            params: {'user': 'text', 'service_pass': 'text'}
        },
        "include_process": {
            title: "Additional Process Flow",
            type: "star",
            color: '#FFAAAA',
            params: {process: 'text'}
        },
        "p_unload": {
            title: "Passengers unloading service",
            type: "square",
            color: '#AAAAFF',
            params: {}
        },
        "p_load": {
            title: "Passengers loading service",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "p_load_finalize": {
            title: "Passengers loaded action",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "p_load_search": {
            title: "Passengers search service",
            type: "square",
            color: '#FFAAAA',
            params: {}
        },
        "hose": {
            title: "Request hose service",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "t_hose": {
            title: "Terminate hose service",
            type: "square",
            color: '#FFAAAA',
            params: {}
        },
        "ladder": {
            title: "Request ladder service",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "t_ladder": {
            title: "Terminate ladder service",
            type: "square",
            color: '#FFAAAA',
            params: {}
        },
        "bus": {
            title: "Request bus service",
            type: "square",
            color: '#AAFFAA',
            params: {}
        },
        "t_bus": {
            title: "Terminate bus service",
            type: "square",
            color: '#FFAAAA',
            params: {}
        }
    };
    this.x2j = new X2JS();
    this.getXmlNodes = function() {
        if (console) { console.log('string -> XML'); }
        this.xmlNodes = this.x2j.xml_str2json(this.file.content);
    };
    this.parseToInternal = function(xmlNodes) {
        var res = {
            ref: {},
            links: {},
            nodes: {array: []}
        };
        var editor = this;
        if (xmlNodes.nodes) {
            if (xmlNodes.nodes.node) {
                var node;
                var xmlNode;
                var links;
                if (typeof xmlNodes.nodes.node.length != "undefined") {
                    for (var i = 0; i < xmlNodes.nodes.node.length; i++) {
                        xmlNode = xmlNodes.nodes.node[i];
                        links = xmlNode.links;
                        if (typeof links != "undefined") {
                            delete xmlNode.links;
                        }
                        node = {
                            links: {array: []}
                        };
                        $.each(['id','x','y','type','state','message','user','service_pass','process'], function(l, k) {
                            if (typeof xmlNode[k] != "undefined") {
                                var el = xmlNode[k];
                                node[k] = editor.desk.utils.trim(el + '');
                            }
                        });
                        node.x = node.x - 0;
                        node.y = node.y - 0;
                        res.links[node.id] = {};
                        res.ref[node.id] = node;
                        if (links && links.link) {
                            var xmlLink;
                            var link;
                            if (typeof links.link.length != "undefined") {
                                for (var k = 0; k < links.link.length; k++) {
                                    xmlLink = links.link[k];
                                    link = {
                                        to: this.desk.utils.trim(xmlLink.to + '')
                                    };
                                    res.links[node.id][link.to] = link;
                                    node.links.array.push({link: link});
                                }
                            } else {
                                xmlLink = links.link;
                                link = {
                                    to: this.desk.utils.trim(xmlLink.to + '')
                                };
                                res.links[node.id][link.to] = link;
                                node.links.array.push({link: link});
                            }
                        }
                        res.nodes.array.push({node: node});
                    }
                } else {
                    xmlNode = xmlNodes.nodes.node;
                    if (typeof xmlNode.links != "undefined") {
                        delete xmlNode.links;
                    }
                    node = {
                        links: {array: []}
                    };
                    $.each(['id','x','y','type','state','message','user','service_pass','process'], function(l, k) {
                        if (typeof xmlNode[k] != "undefined") {
                            var el = xmlNode[k];
                            node[k] = editor.desk.utils.trim(el + '');
                        }
                    });
                    node.x = node.x - 0;
                    node.y = node.y - 0;
                    res.links[node.id] = {};
                    res.ref[node.id] = node;
                    res.nodes.array.push({node: node});
                }
            }
        }
        return res;
    };
    this.convertToInternal = function() {
        if (console) { console.log('XML -> Internal'); }
        this.internalNodes = this.parseToInternal(this.xmlNodes);
    };
    this.convertToXml = function() {
        if (console) { console.log('Internal -> XML -> string'); }
        return this.x2j.json2xml_str({nodes: this.internalNodes.nodes});
    };
    this.convertToVisual = function() {
        if (console) { console.log('Internal -> Visual'); }
        var nodes = this.internalNodes;
        var res = [];
        for (var i = 0; i < nodes.nodes.array.length; i++) {
            var internalNode = nodes.nodes.array[i].node;
            var node = {};
            node.id = internalNode.id;
            node.name = "#" + internalNode.id;
            node.data = {
                "$color": this.nodeTypes[internalNode.type].color,
                "$type": this.nodeTypes[internalNode.type].type
            };
            node.adjacencies = [];
            for (var k = 0; k < internalNode.links.array.length; k++) {
                var internalLink = internalNode.links.array[k].link;
                node.adjacencies.push({
                    nodeTo: internalLink.to,
                    data: {
                        "$type": 'arrowhyperline'
                    }
                });
            }
            res.push(node);
        }
        try { throw new Error(); } catch(e) {}//breakpoint
        this.visualNodes = res;
    };
    this.compile = function(callback) {
        this.compileProcess(function(res) {
            callback(res);
        }, undefined, function(err) {
            callback(err);
        });
    };
    this.showCompilation = function(results) {
        var d = $("<div title='Compilation'>");
        var perl = $("<textarea>");
        perl.css('width', '100%');
        perl.css('height', '400px');
        perl.val(results);
        d.append(perl);
        $("body").append(d);
        d.dialog({
            resizable: false,
            width: 600,
            height:400,
            modal: true
        });
    };
    this.restorePositions = function(animate) {
        var editor = this;
        this.jit.graph.eachNode(function(n) {
            var p;
            if (animate) {
                p = n.getPos("start");
                p.x = 0;
                p.y = 0;
                p = n.getPos("end");
                p.x = editor.internalNodes.ref[n.id].x - 0;
                p.y = editor.internalNodes.ref[n.id].y - 0;
            } else {
                p = n.getPos();
                p.x = editor.internalNodes.ref[n.id].x - 0;
                p.y = editor.internalNodes.ref[n.id].y - 0;
            }
        });
        this.jit.plot();
        if (animate) {
            this.jit.animate({
                modes: ['linear'],
                transition: $jit.Trans.Elastic.easeOut,
                duration: 2500
            });
        }
    };
    this.reloadJSON = function(forceReread, animate) {
        this.parse(forceReread);
        if (this.visualNodes.length > 0) {
            this.jit.loadJSON(this.visualNodes);
            this.visualNodes = undefined;
            this.restorePositions(animate);
        }
    };
    this.modified = function(value) {
        var needReread = false;
        if (!value) {
            value = this.convertToXml();
        } else {
            needReread = true;
        }
        this.desk.fileObj.notifyModification(this.project, this, this.file, value);
        if (needReread) {
            this.reloadJSON(true);
        }
    };
    this.selectNewId = function() {
        var maxId = 0;
        for(var i = 0; i < this.internalNodes.nodes.array.length; i++) {
            if (this.internalNodes.nodes.array[i].node.id - 0 > maxId) {
                maxId = this.internalNodes.nodes.array[i].node.id - 0;
            }
        }
        return (maxId + 1) + '';
    };
    this.newNode = function(x, y) {
        var node = {
            id: this.selectNewId(),
            x: x,
            y: y,
            type: "takeoff",
            links: {array: []}
        };
        this.internalNodes.nodes.array.push({node: node});
        this.internalNodes.ref[node.id] = node;
        this.internalNodes.links[node.id] = {};
        var visualNode = {
                id: node.id,
                name: "#" + node.id,
                data: {
                    "$x": node.x,
                    "$y": node.y,
                    "$color": this.nodeTypes[node.type].color,
                    "$type": this.nodeTypes[node.type].type
                },
                adjacencies: []
            };
        visualNode = this.jit.graph.addNode(visualNode);
        var p = visualNode.getPos();
        p.x = this.internalNodes.ref[visualNode.id].x - 0;
        p.y = this.internalNodes.ref[visualNode.id].y - 0;
        /*var p = visualNode.getPos("start");
        p.x = 0;
        p.y = 0;
        p = visualNode.getPos("end");
        p.x = this.internalNodes.ref[visualNode.id].x - 0;
        p.y = this.internalNodes.ref[visualNode.id].y - 0;*/
        if (!this.jit.root) {
            this.jit.root = visualNode.id;
        }
        this.jit.plot();
        this.modified();
        /*this.jit.animate({
            modes: ['linear'],
            transition: $jit.Trans.Elastic.easeOut,
            duration: 2500
        });*/
    };
    this.hideNode = function(visualNode) {
        visualNode.setData('alpha', 0, 'end');
        visualNode.eachAdjacency(function(adj) {
            adj.setData('alpha', 0, 'end');
        });
        this.jit.fx.animate({
            modes: ['node-property:dim:alpha',
                'edge-property:lineWidth:color:alpha'],
            duration: 500
        });
    };
    this.deleteNode = function(visualNode) {
        delete this.internalNodes.ref[visualNode.id];
        delete this.internalNodes.links[visualNode.id];
        for (var i = 0; i < this.internalNodes.nodes.array.length; i++) {
            if (this.internalNodes.nodes.array[i].node.id == visualNode.id) {
                this.internalNodes.nodes.array.splice(i ,1);
                i--;
            } else {
                for (var k = 0; k < this.internalNodes.nodes.array[i].node.links.array.length; k++) {
                    if (this.internalNodes.nodes.array[i].node.links.array[k].link.to == visualNode.id) {
                        delete this.internalNodes.links[this.internalNodes.nodes.array[i].node.id][visualNode.id];
                        this.internalNodes.nodes.array[i].node.links.array.splice(k, 1);
                        k--;
                    }
                }
            }
        }
        if (this.jit.root == visualNode.id) {
            if (this.internalNodes.nodes.array.length > 0) {
                this.jit.root = this.internalNodes.nodes.array[0].node.id;
            } else {
                this.jit.root = undefined;
            }
        }
        this.jit.graph.removeNode(visualNode.id);
        this.jit.plot();
        this.modified();
        this.toggleNodeSelect(undefined);
    };
    this.hideEdge = function(visualEdge) {
        visualEdge.setData('alpha', 0, 'end');
        this.jit.fx.animate({
            modes: ['edge-property:lineWidth:color:alpha'],
            duration: 500
        });
    };
    this.deleteEdge = function(visualEdge) {
        delete this.internalNodes.links[visualEdge.nodeFrom.id][visualEdge.nodeTo.id];
        var internalNode = this.internalNodes.ref[visualEdge.nodeFrom.id];
        for (var k = 0; k < internalNode.links.array.length; k++) {
            if (internalNode.links.array[k].link.to == visualEdge.nodeTo.id) {
                internalNode.links.array.splice(k, 1);
                k--;
            }
        }
        this.jit.graph.removeAdjacence(visualEdge.nodeFrom.id, visualEdge.nodeTo.id);
        this.jit.plot();
        this.modified();
        this.toggleEdgeSelect(undefined);
    };

    this.configureZone = undefined;
    this.selectedVisualNode = undefined;
    this.selectedVisualEdge = undefined;
    this.showConfigureEdge = function(visualEdge) {
        var editor = this;
        this.hideConfigure();
        this.configureZone = $("<div>");
        if (this.addNode) {
            this.addNode.parent().append(this.configureZone);
        }
        if (this.file.is_writable) {
            var s = $("<div>");
            s.attr("title", "Delete edge");
            s.addClass("icon-button delete-edge-icon-button");
            s.click(function() {
                s.attr("disabled", "disabled");
                editor.hideEdge(visualEdge);
                editor.deleteEdge(visualEdge);
            });
            this.configureZone.append(s);
        }
    };
    this.showConfigureNode = function(visualNode) {
        var editor = this;

        this.hideConfigure();
        this.configureZone = $("<div>");
        if (this.addNode) {
            this.addNode.parent().append(this.configureZone);
        }
        var nodeType = $("<select></select>");
        $.each(this.nodeTypes, function(i, el) {
            nodeType.append("<option>" + i + "</option>");
        });
        if (!this.file.is_writable) {
            nodeType.prop("disabled", true);
        }

        this.configureZone.append("<span>Type&nbsp;</span>");
        this.configureZone.append(nodeType);
        this.configureZone.append("<span class='others'></span>");

        nodeType.val(this.internalNodes.ref[visualNode.id].type);

        var nodeParams = {};

        var changeLayout = function() {
            editor.configureZone.find('.others').children().remove();
            nodeParams = {};
            $.each(editor.nodeTypes[editor.internalNodes.ref[visualNode.id].type].params, function(key, type) {
                var nodeParam;
                if (type == 'text') {
                    nodeParam = $("<input type='text'>");
                }
                nodeParams[key] = nodeParam;
                if (!editor.file.is_writable) {
                    nodeParam.prop("readonly", true);
                }
                editor.configureZone.find('.others').append("<span>" + key + "</span>");
                editor.configureZone.find('.others').append(nodeParam);
            });
            $.each(nodeParams, function(key, el) {
                if (typeof editor.internalNodes.ref[visualNode.id][key] != "undefined") {
                    el.val(editor.internalNodes.ref[visualNode.id][key]);
                } else {
                    el.val('');
                }
                if (editor.file.is_writable) {
                    el.bind('keyup change', function() {
                        editor.internalNodes.ref[visualNode.id][key] = $(this).val();
                        editor.jit.plot();
                        editor.modified();
                    });
                    if (key == 'process') {
                        el.autocomplete({
                            source: editor.availableProcessFiles
                        });
                    }
                }
            });
        };

        changeLayout();

        if (this.file.is_writable) {
            var s = $("<div>");
            s.attr("title", "Delete node");
            s.addClass("icon-button delete-node-icon-button");
            s.click(function() {
                s.attr("disabled", "disabled");
                editor.hideNode(visualNode);
                editor.deleteNode(visualNode);
            });
            this.configureZone.append(s);
        }

        if (this.file.is_writable) {
            nodeType.bind('change', function() {
                editor.internalNodes.ref[visualNode.id].type = $(this).val();
                visualNode.data["$type"] = editor.nodeTypes[editor.internalNodes.ref[visualNode.id].type].type;
                visualNode.data["$color"] = editor.nodeTypes[editor.internalNodes.ref[visualNode.id].type].color;
                editor.jit.plot();
                editor.modified();
                changeLayout();
            });
        }
    };
    this.hideConfigure = function() {
        if (this.configureZone) {
            this.configureZone.remove();
            this.configureZone = undefined;
        }
    };
    this.toggleEdgeSelect = function(visualEdge) {
        this.hideConfigure();
        var editor = this;
        this.jit.graph.eachNode(function(n) {
            n.setData('dim', editor.nodeRadius, 'end');
            n.eachAdjacency(function(adj) {
                if (!visualEdge || adj.nodeFrom.id != visualEdge.nodeFrom.id || adj.nodeTo.id != visualEdge.nodeTo.id) delete adj.selected;
                adj.setDataset('end', {
                    lineWidth: 2,
                    color: '#23a4ff'
                });
            });
        });
        if (visualEdge) {
            if(!visualEdge.selected) {
                this.showConfigureEdge(visualEdge);
                this.selectedVisualEdge = visualEdge;
                visualEdge.selected = true;
                visualEdge.setDataset('end', {
                    lineWidth: 6,
                    color: '#36acfb'
                });
                visualEdge.nodeFrom.setData('dim', editor.nodeActiveRadius, 'end');
                visualEdge.nodeTo.setData('dim', editor.nodeActiveRadius, 'end');
            } else {
                this.selectedVisualEdge = undefined;
                delete visualEdge.selected;
            }
        } else {
            this.selectedVisualEdge = undefined;
        }
        if (this.selectedVisualNode) {
            delete this.selectedVisualNode.selected;
            this.selectedVisualNode = undefined;
        }
        this.jit.fx.animate({
            modes: ['node-property:dim:alpha',
                'edge-property:lineWidth:color:alpha'],
            duration: 500
        });
    };
    this.toggleNodeSelect = function(visualNode) {
        this.hideConfigure();
        var editor = this;
        this.jit.graph.eachNode(function(n) {
            if(!visualNode || n.id != visualNode.id) delete n.selected;
            n.setData('dim', editor.nodeRadius, 'end');
            n.eachAdjacency(function(adj) {
                adj.setDataset('end', {
                    lineWidth: 2,
                    color: '#23a4ff'
                });
            });
        });
        if (visualNode) {
            if(!visualNode.selected) {
                this.showConfigureNode(visualNode);
                this.selectedVisualNode = visualNode;
                visualNode.selected = true;
                visualNode.setData('dim', editor.nodeActiveRadius, 'end');
                visualNode.eachAdjacency(function(adj) {
                    adj.setDataset('end', {
                        lineWidth: 6,
                        color: '#36acfb'
                    });
                });
            } else {
                this.selectedVisualNode = undefined;
                delete visualNode.selected;
            }
        } else {
            this.selectedVisualNode = undefined;
        }
        if (this.selectedVisualEdge) {
            delete this.selectedVisualEdge.selected;
            this.selectedVisualEdge = undefined;
        }
        this.jit.fx.animate({
            modes: ['node-property:dim:alpha',
                'edge-property:lineWidth:color:alpha'],
            duration: 500
        });
    };
    this.handleEdgeCreate = function(visualNode) {
        if (!this.selectedVisualNode) {
            this.toggleNodeSelect(visualNode);
        } else {
            var i = 0;
            for(; i < this.internalNodes.ref[this.selectedVisualNode.id].links.array.length; i++) {
                if (this.internalNodes.ref[this.selectedVisualNode.id].links.array[i].link.to == visualNode.id) {
                    break;
                }
            }
            if (i >= this.internalNodes.ref[this.selectedVisualNode.id].links.array.length) {
                var internalLink = {to: visualNode.id, activation: 1, stream: 1};
                this.internalNodes.ref[this.selectedVisualNode.id].links.array.push({link: internalLink});
                this.internalNodes.links[this.selectedVisualNode.id][internalLink.to] = internalLink;
                var visualEdge = {
                        "$type": 'arrowhyperline'
                    };
                this.jit.graph.addAdjacence(this.selectedVisualNode, visualNode, visualEdge);
                this.jit.plot();
                this.modified();
            }
            this.toggleNodeSelect(undefined);
            this.toggleEdgeSelect(undefined);
        }
    };
    this.prepareLine = function(fromNode, toNode) {
        var res = {
            'arrow': {
                'vect': undefined,
                'end': undefined
            },
            'line': {
                'type': undefined,
                'params': []
            },
            'textPos': undefined,
            'clickZone': {
                'type': undefined,
                'arc': {
                    'sector': {
                        'centerPos': undefined,
                        'startPos': undefined,
                        'endPos': undefined,
                        'reverse': false
                    },
                    'ring': {
                        'radiusStart': undefined,
                        'radiusEnd': undefined
                    }
                },
                'line': {
                    'lineStart': undefined,
                    'lineEnd': undefined,
                    'lineWidth': undefined
                }
            }
        };

        var nodeSize = this.nodeRadius;
        var arrowSize = 12;
        var clickSize = 4;

        // offset by nodeSize
        var from = fromNode.getPos();
        var to = toNode.getPos();

        var vect;
        var centerOfCircle, angleBegin, angleEnd, s, vectFromCenter;
        // self link
        if (fromNode.id == toNode.id) {
            var d = Math.cos(Math.PI / 4) * nodeSize;
            from = new $jit.Complex(from.x - d, from.y - d);
            to = new $jit.Complex(to.x + d, to.y - d);

            centerOfCircle = {x: from.x + d, y: from.y - d, ratio: nodeSize};
            // draw line
            angleBegin = Math.atan2(from.y - centerOfCircle.y, from.x - centerOfCircle.x);
            angleEnd = Math.atan2(to.y - centerOfCircle.y, to.x - centerOfCircle.x);
            s = !sense(angleBegin, angleEnd);
            res.line.type = 'arc';
            res.line.params = [centerOfCircle.x, centerOfCircle.y, centerOfCircle.ratio, angleBegin, angleEnd, s];

            res.clickZone.type = 'arc';
            res.clickZone.arc.sector.centerPos = centerOfCircle;
            res.clickZone.arc.sector.startPos = (s ? from : to);
            res.clickZone.arc.sector.endPos = (s ? to : from);
            res.clickZone.arc.sector.reverse = true;
            res.clickZone.arc.ring.radiusStart = centerOfCircle.ratio - clickSize;
            res.clickZone.arc.ring.radiusEnd = centerOfCircle.ratio + clickSize;

            // count arrow
            vectFromCenter = new $jit.Complex(Math.cos(angleEnd) * centerOfCircle.ratio, Math.sin(angleEnd) * centerOfCircle.ratio);
            vect = new $jit.Complex(vectFromCenter.y / 2, -vectFromCenter.x / 2);
            n = 1 / vect.norm();
            vect.$scale(n * arrowSize * (s ? 1 : -1));
            res.arrow.vect = vect;
            res.arrow.end = to;

            if (angleBegin < 0) {
                angleBegin += Math.PI * 2;
            }
            if (angleEnd < 0) {
                angleEnd += Math.PI * 2;
            }
            if (Math.abs(Math.abs(angleEnd - angleBegin) - Math.PI) < 1e-4) {
                angleEnd += 1e-2;
            }
            // count text
            vectFromCenter = new $jit.Complex(Math.cos((angleBegin + angleEnd) / 2 + Math.PI) * centerOfCircle.ratio, Math.sin((angleBegin + angleEnd) / 2 + Math.PI) * centerOfCircle.ratio);
            vectFromCenter.$add(centerOfCircle);
            res.textPos = vectFromCenter;

        } else {// link to another node
            vect = new $jit.Complex(to.x - from.x, to.y - from.y);

            if (vect.norm() < nodeSize * 2) {// too close
                return res;
            }
            var n = 1 / vect.norm();
            vect.x *= n * nodeSize;
            vect.y *= n * nodeSize;
            from = new $jit.Complex(from.x + vect.x, from.y + vect.y);
            to = new $jit.Complex(to.x - vect.x, to.y - vect.y);

            centerOfCircle = computeArcThroughTwoPoints(from, to);
            if (typeof centerOfCircle.ratio == 'undefined') {
                // draw line
                res.line.type = 'line';
                res.line.params = [[from.x, from.y], [to.x, to.y]];

                res.clickZone.type = 'line';
                res.clickZone.line.lineStart = from;
                res.clickZone.line.lineEnd = to;
                res.clickZone.line.lineWidth = clickSize;

                // count arrow
                vect = new $jit.Complex(to.x - from.x, to.y - from.y);
                n = 1 / vect.norm();
                vect.$scale(n * arrowSize);
                res.arrow.vect = vect;
                res.arrow.end = to;

                // count text
                res.textPos = new $jit.Complex((from.x + to.x) / 2, (from.y + to.y) / 2);
            } else {
                // draw line
                angleBegin = Math.atan2(from.y - centerOfCircle.y, from.x - centerOfCircle.x);
                angleEnd = Math.atan2(to.y - centerOfCircle.y, to.x - centerOfCircle.x);
                s = sense(angleBegin, angleEnd);
                res.line.type = 'arc';
                res.line.params = [centerOfCircle.x, centerOfCircle.y, centerOfCircle.ratio, angleBegin, angleEnd, s];

                res.clickZone.type = 'arc';
                res.clickZone.arc.sector.centerPos = centerOfCircle;
                res.clickZone.arc.sector.startPos = (s ? to : from);
                res.clickZone.arc.sector.endPos = (s ? from : to);
                res.clickZone.arc.ring.radiusStart = centerOfCircle.ratio - clickSize;
                res.clickZone.arc.ring.radiusEnd = centerOfCircle.ratio + clickSize;

                // count arrow
                vectFromCenter = new $jit.Complex(Math.cos(angleEnd) * centerOfCircle.ratio, Math.sin(angleEnd) * centerOfCircle.ratio);
                vect = new $jit.Complex(vectFromCenter.y / 2, -vectFromCenter.x / 2);
                n = 1 / vect.norm();
                vect.$scale(n * arrowSize * (s ? 1 : -1));
                res.arrow.vect = vect;
                res.arrow.end = to;

                if (angleBegin < 0) {
                    angleBegin += Math.PI * 2;
                }
                if (angleEnd < 0) {
                    angleEnd += Math.PI * 2;
                }
                if (Math.abs(Math.abs(angleEnd - angleBegin) - Math.PI) < 1e-4) {
                    angleEnd += 1e-2;
                }
                // count text
                vectFromCenter = new $jit.Complex(to.x - from.x, to.y - from.y);
                vectFromCenter.$scale(0.5);
                vectFromCenter.$add(from);
                vectFromCenter.$scale(-1);
                vectFromCenter.$add(centerOfCircle);
                vectFromCenter.$scale(-1);
                n = centerOfCircle.ratio / vectFromCenter.norm();
                vectFromCenter.$scale(n);
                vectFromCenter.$add(centerOfCircle);
                res.textPos = vectFromCenter;
            }
        }

        return res;

        function computeArcThroughTwoPoints(p1, p2, radius_mult) {
            if (!radius_mult) {
                radius_mult = 1;
            }
            var A = p2.y - p1.y;
            var B = -(p2.x - p1.x);
            var C = -A * p1.x - B * p1.y;
            var M = Math.sqrt(A * A + B * B);
            A /= M;
            B /= M;
            C /= M;

            var x = (p2.x + p1.x) / 2; //middle
            var y = (p2.y + p1.y) / 2;
            var ratio = (M + Math.sqrt(M)) * radius_mult;

            x += A * Math.sqrt(ratio * ratio - M * M / 4);
            y += B * Math.sqrt(ratio * ratio - M * M / 4);

            return {
                    x: x,
                    y: y,
                    ratio: ratio
                };
        }
        function sense(angleBegin, angleEnd) {
            return (angleBegin < angleEnd)? (angleBegin + Math.PI <= angleEnd) : (angleEnd + Math.PI > angleBegin);
        }
    };
    this.clickEdge = function(fromNode, toNode, pos) {
        var conf = this.prepareLine(fromNode, toNode);
        try { throw new Error(); } catch(e) {}//breakpoint
        if (conf.clickZone.type == 'arc') {//TODO:called only for direct edges
            return this.utils.dist2(conf.textPos, pos) < this.utils.sqr(conf.clickZone.arc.ring.radiusEnd - conf.clickZone.arc.ring.radiusStart);
        }
        if (conf.clickZone.type == 'line') {
            return this.utils.distToSegment(pos, conf.clickZone.line.lineStart, conf.clickZone.line.lineEnd) < conf.clickZone.line.lineWidth;
        }
        return false;
    };
    this.utils = {
        sqr: function(x) { return x * x; },
        dist: function(v, w) { return Math.sqrt(this.dist2(v, w)); },
        dist2: function(v, w) { return this.sqr(v.x - w.x) + this.sqr(v.y - w.y); },
        distToSegmentSquared: function(p, v, w) {
            var lq = this.dist2(v, w);
            if (lq == 0) { return this.dist2(p, v); }
            var t = ((p.x - v.x) * (w.x - v.x) + (p.y - v.y) * (w.y - v.y)) / lq;
            if (t < 0) { return this.dist2(p, v); }//start side
            if (t > 1) { return this.dist2(p, w); }//end side
            return this.dist2(p, { x: v.x + t * (w.x - v.x),
                              y: v.y + t * (w.y - v.y) });
        },
        distToSegment: function(p, v, w) { return Math.sqrt(this.distToSegmentSquared(p, v, w)); }
    };
    this.drawEdge = function(fromNode, toNode, adj, canvas) {
        var ctx = canvas.getCtx();
        var conf = this.prepareLine(fromNode, toNode);

        if (conf.line.type == 'arc') {
            ctx.beginPath();
            ctx.arc.apply(ctx, conf.line.params);
            ctx.stroke();
        }
        if (conf.line.type == 'line') {
            ctx.beginPath();
            ctx.moveTo.apply(ctx, conf.line.params[0]);
            ctx.lineTo.apply(ctx, conf.line.params[1]);
            ctx.stroke();
        }

        /* arrow */
        if (conf.arrow.end) {
            var intermediatePoint = new $jit.Complex(conf.arrow.end.x - conf.arrow.vect.x, conf.arrow.end.y - conf.arrow.vect.y);
            var normal = new $jit.Complex(-conf.arrow.vect.y / 2, conf.arrow.vect.x / 2);
            var v1 = intermediatePoint.add(normal);
            var v2 = intermediatePoint.$add(normal.$scale(-1));

            ctx.beginPath();
            ctx.moveTo(v1.x, v1.y);
            ctx.lineTo(v2.x, v2.y);
            ctx.lineTo(conf.arrow.end.x, conf.arrow.end.y);
            ctx.closePath();
            ctx.fill();
        }
    };
};
FlightProcessEditor.prototype.parse = function(forceReread) {
    if (forceReread) {
        this.getXmlNodes();
        this.convertToInternal();
    }
    this.convertToVisual();
};
FlightProcessEditor.prototype.activated = function(editorZone) {
    this.getProcessFilesInLibraries(function(data) {
        this.availableProcessFiles = data;
    }, this);
    var editor = this;
    editorZone.children().remove();
    var id = this.desk.utils.generateRandom(12);
    this.el = $('<div>');
    this.el.css('width', '100%');
    this.el.css('height', '100%');
    this.el.css('background-color', '#555555');
    var pallete = $('<div>');
    pallete.attr('class', 'pallete');
    pallete.css('height', '40px');
    this.el.append(pallete);
    var canv = $('<div id="' + id + '" class="flight-canvas">');
    canv.css('width', '100%');
    canv.css('height', '100%');
    this.el.append(canv);
    editorZone.append(this.el);

    this.addNode = $("<div>");
    if (this.file.is_writable) {
        this.addNode.attr("title", "Add node");
        this.addNode.addClass("icon-button add-node-icon-button");
        this.addNode.click(this, function(event) {
            var pos = {x: (-1) * event.data.jit.canvas.translateOffsetX, y: (-1) * event.data.jit.canvas.translateOffsetY};
            pos.x += Math.random() * 30 - 15;
            pos.y += Math.random() * 30 - 15;
            editor.newNode(pos.x, pos.y);
        });
    }
    pallete.append(this.addNode);

    this.startCompile = $("<div>");
    this.startCompile.attr("title", "Start compilation");
    this.startCompile.addClass("icon-button compile-icon-button");
    this.startCompile.click(this, function(event) {
        event.data.compile(function(message) { event.data.showCompilation(message); });
    });
    pallete.append(this.startCompile);

    $jit.ForceDirected.Plot.EdgeTypes.implement({
        'arrowhyperline': {
            'render': function(adj, canvas, animating) {
                if (adj.getData('alpha') > 0) {
                    editor.drawEdge(adj.nodeFrom, adj.nodeTo, adj, canvas);
                }
            },
            'contains': function(adj, pos) {
                if (adj.getData('alpha') > 0) {
                    return editor.clickEdge(adj.nodeFrom, adj.nodeTo, pos);
                }
            }
        }
    });
    this.jit = new $jit.ForceDirected({
        injectInto: id,
        Label: {
            type: 'Native',
            size: 11,
            color: '#555'
        },
        //Enable zooming and panning
        //with scrolling and DnD
        Navigation: {
            enable: true,
            type: 'Native',
            //Enable panning events only if we're dragging the empty
            //canvas (and not a node).
            panning: 'avoid nodes',
            zooming: 10 //zoom speed. higher is more sensible
        },
        // Change node and edge styles such as
        // color and width.
        // These properties are also set per node
        // with dollar prefixed data-properties in the
        // JSON structure.
        Node: {
            overridable: true,
            color: '#AAFFAA',
            dim: editor.nodeRadius
        },
        Edge: {
            overridable: true,
            color: '#23A4FF',
            lineWidth: 2
        },
        // Add node events
        Events: {
            enable: true,
            enableForEdges: true,
            type: 'Native',
            //Change cursor style when hovering a node
            onMouseEnter: function() {
                if (editor.file.is_writable) {
                    editor.jit.canvas.getElement().style.cursor = 'move';
                } else {
                    editor.jit.canvas.getElement().style.cursor = 'pointer';
                }
            },
            onMouseLeave: function() {
                editor.jit.canvas.getElement().style.cursor = '';
            },
            //Update node positions when dragged
            onDragMove: function(visualNode, eventInfo, e) {
                if (editor.file.is_writable) {
                    if (!visualNode.nodeFrom) {
                        var pos = eventInfo.getPos();
                        visualNode.pos.setc(pos.x, pos.y);
                        editor.internalNodes.ref[visualNode.id].x = pos.x;
                        editor.internalNodes.ref[visualNode.id].y = pos.y;
                        editor.modified();
                        editor.jit.plot();
                    }
                }
            },
            onClick: function(visualNode, eventInfo, e) {
                if (editor.selectedVisualNode) {
                    var pos = eventInfo.getPos();
                    var done = false;
                    editor.jit.graph.eachNode(function(n) {
                        if (done) {
                            return;
                        }
                        var selectedPos = n.pos.getc(true);
                        var cleanPos = {x: selectedPos.x, y: selectedPos.y - 50};
                        if (editor.utils.dist(pos, cleanPos) < 20) {
                            editor.handleEdgeCreate(n);
                            done = true;
                        }
                    });
                }
                if (visualNode) {
                    if (visualNode.nodeFrom) {
                        editor.toggleEdgeSelect(visualNode);
                    } else {
                        editor.toggleNodeSelect(visualNode);
                    }
                } else {
                    editor.toggleEdgeSelect(undefined);
                    editor.toggleNodeSelect(undefined);
                }
            },
            //Implement the same handler for touchscreens
            onTouchMove: function(visualNode, eventInfo, e) {
                $jit.util.event.stop(e); //stop default touchmove event
                this.onDragMove(visualNode, eventInfo, e);
            }
        },
        //Number of iterations for the FD algorithm
        iterations: 200,
        //Edge length
        levelDistance: 130,
        onAfterPlotNode: function(visualNode) {
            if (visualNode.getData('alpha') > 0) {
                var ctx = editor.jit.canvas.getCtx();
                var pos = visualNode.pos.getc(true);

                ctx.font = visualNode.getLabelData('style') + ' ' + visualNode.getLabelData('size') + 'px ' + visualNode.getLabelData('family');
                ctx.textAlign = visualNode.getLabelData('textAlign');
                ctx.fillStyle = ctx.strokeStyle = visualNode.getLabelData('color');
                ctx.textBaseline = visualNode.getLabelData('textBaseline');
                ctx.fillText("(" + editor.internalNodes.ref[visualNode.id].type + ")", pos.x, pos.y + visualNode.getData("height") / 2 - visualNode.getLabelData('size'));

                if (editor.selectedVisualNode) {
                    ctx.drawImage(editor.preloadedEdgeImage, 0, 0, 40, 40, pos.x - 20, pos.y - 70, 40, 40);
                }
            }
        },
        onComplete: function() {
            editor.resizerInterval = window.setInterval(function() {
                    if(editor.el && editor.el.children()[1].id == canv[0].id) {
                        if ($(editor.jit.canvas.getElement()).width() != canv.width() || $(editor.jit.canvas.getElement()).height() != canv.height() && editor.internalNodes.nodes.array.length > 0) {
                            editor.jit.canvas.resize(canv.width(), canv.height());
                            editor.convertToVisual();
                            editor.restorePositions();
                        }
                    } else {
                        window.clearInterval(editor.resizerInterval);
                    }
                }, 1000);
        }
     });

    this.toggleNodeSelect(undefined);
    this.toggleEdgeSelect(undefined);

    // load JSON data.
    this.reloadJSON(true, true);
    editor.jit.animate({
        modes: ['linear'],
        transition: $jit.Trans.Elastic.easeOut,
        duration: 2500
    });
};
FlightProcessEditor.prototype.fileChanged = function(modification) {
    console.log("New! " + modification.newFile.content);
    modification.newFile.content = this.file.content;
    this.file = modification.newFile;
    if (this.jit && modification.source != this) {
        this.reloadJSON(true);
    }
};
FlightProcessEditor.editorName = function() { return "FlightProcess"; };
FlightProcessEditor.produce = function(desk, project, file) {
    return new FlightProcessEditor(desk, project, file);
};
FlightProcessEditor.id = "FlightProcessEditor";
/* plugging into IDE */
function getEditorFactory() {
    $("head").append('<link href="' + Desk.current.server + '/api/files/browse/flightprocess_editor/flightprocess-editor-styles.css" rel="stylesheet" type="text/css" />');
    return FlightProcessEditor;
}