/**
 * User: progsdi
 * Date: 27.06.12
 * Time: 16:21
 */
var CommonProjectRenderer = function(desk, projectNode, fileList) {
    if (projectNode.type != Desk.current.projectObj.nodes.PROJECT) {
        throw new Error("CommonProjectRenderer supports only Project nodes");
    }
    this.desk = desk;
    this.projectNode = projectNode;
    this.fileList = fileList;
    this.refresh();
};
CommonProjectRenderer.prototype.filesChanged = function(fileList) {
    this.fileList = fileList;
    this.refresh();
};
CommonProjectRenderer.prototype.currentChanged = function(node) {
    this.desk.context.contextChange(CommonProjectRenderer.STATE_FILE, node);
    return true;
};
CommonProjectRenderer.prototype.addFileToTree = function(parent, file) {
    var treeFile = {};
    treeFile.label = file.name;
    treeFile.data = {data: file, parentData: parent.data, renderer: this};
    treeFile.type = CommonProjectRenderer.NODE_FILE;
    file.data = treeFile;
    parent.children.push(treeFile);
};
CommonProjectRenderer.prototype.addFileToList = function(file) {
    this.fileList.push(file);
    this.refresh();
};
CommonProjectRenderer.prototype.deleteFileFromList = function(file) {
    for (var i = 0; i < this.fileList.length; i++) {
        if (this.fileList[i].id == file.id) {
            this.fileList.splice(i, 1);
            i--;
        }
    }
    this.refresh();
};
CommonProjectRenderer.prototype.refresh = function() {
    this.projectNode.children = [];
    for (var i = 0; i < this.fileList.length; i++) {
        this.addFileToTree(this.projectNode, this.fileList[i]);
        this.desk.fileObj.startEditor(this.projectNode.data.data, this.fileList[i], undefined, true);
    }
    this.desk.projectObj.reloadData();
};
CommonProjectRenderer.NODE_FILE = "CommonProjectRenderer-file";
CommonProjectRenderer.STATE_FILE = "CommonProjectRenderer.file";
CommonProjectRenderer.addFile = function(element, menu) {
    element.data.renderer.desk.fileObj.addFileState(element.data.renderer, element, menu, element.data.renderer.projectNode);
};
CommonProjectRenderer.deleteFile = function(element, menu) {
    element.data.renderer.desk.fileObj.deleteFileState(element.data.renderer, element, menu, element.data.renderer.projectNode);
};
CommonProjectRenderer.editFile = function(element, menu) {
    element.data.renderer.desk.fileObj.startEditor(element.data.parentData.data, element.data.data);
};
CommonProjectRenderer.selectFileEditor = function(element, menu) {
    element.data.renderer.desk.fileObj.showSelectFileEditor(element.data.renderer, element, menu, element.data.renderer.projectNode);
};

CommonProjectRenderer.projectProperties = function(element, menu, propertiesObj) {
    Desk.current.projectObj.projectSelectState(element.data.renderer, element, menu, propertiesObj, element.data.renderer.projectNode);
};
CommonProjectRenderer.emptyProperties = function(element, menu, propertiesObj) {
    Desk.current.propertiesObj.cleanProperties();
};

var once = true;

CommonProjectRenderer.produce = function(desk, projectNode, fileList) {
    if (once) {
        Desk.registerContext(Desk.Common.states.PROJECT, CommonProjectRenderer.projectProperties, undefined);
        Desk.registerContext(Desk.Common.states.ANY, CommonProjectRenderer.emptyProperties, undefined);
        Desk.registerContext(Desk.Common.states.PROJECT, CommonProjectRenderer.addFile, undefined);
        Desk.registerContext(CommonProjectRenderer.STATE_FILE, CommonProjectRenderer.addFile, undefined);
        Desk.registerContext(CommonProjectRenderer.STATE_FILE, CommonProjectRenderer.deleteFile, undefined);
        Desk.registerContext(CommonProjectRenderer.STATE_FILE, CommonProjectRenderer.editFile, undefined);
        Desk.registerContext(CommonProjectRenderer.STATE_FILE, CommonProjectRenderer.selectFileEditor, undefined);
        once = false;
    }

    return new CommonProjectRenderer(desk, projectNode, fileList);
};
CommonProjectRenderer.id = "CommonProjectRenderer";
/* plugging into IDE */
function getRendererFactory() {
    $("head").append('<link href="' + Desk.current.server + '/api/files/browse/project_renderer/project-renderer.css" rel="stylesheet" type="text/css" />');
    return CommonProjectRenderer;
}