# coding=UTF-8
import random
from django.utils import simplejson
from lxml import etree
from web import models

__author__ = 'IoS'

class CompileService(object):
    step_limit = 150
    by_type = {
        "none": lambda compiler, node, params, steps: params,
        "takeoff": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "land": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "crash": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "b_unload": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "b_load": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "log": lambda compiler, node, params, steps: CompileService.log(node, params),
        "registration_request": lambda compiler, node, params, steps: CompileService.registration(node, params),
        "include_process": lambda compiler, node, params, steps: compiler.__include(node, params),
        "p_unload": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "p_load": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "p_load_finalize": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "p_load_search": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "hose": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "t_hose": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "ladder": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "t_ladder": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "bus": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
        "t_bus": lambda compiler, node, params, steps: CompileService.add_count(node, params, node["type"]),
    }

    @classmethod
    def add_count(cls, node, params, param_name):
        params[param_name] = params.get(param_name, 0) + 1
        return params

    @classmethod
    def extend(cls, node, params, more_params):
        return dict([(key, val) for key, val in params.iteritems()] + [(key, val) for key, val in more_params.iteritems()])

    @classmethod
    def log(cls, node, params):
        params['logger_res'] = params.get('logger_res', '') + '|node#' + node.get('id', '?') + ':[' + node.get('state', '') + ']' + node.get('message', '') + " Whole state: " + simplejson.dumps(params)
        return params

    @classmethod
    def registration(cls, node, params):
        params['logger_res'] = params.get('logger_res', '') + '|node#' + node.get('id', '?') + ':Register ' + node.get('user', '') + ' (' + node.get('service_pass', '') + ')'
        return params

    def launch(self, file, project_state, project, start_params):
        return self.__execute(file.content, start_params, 0)

    class Graph(object):
        def __init__(self):
            self.nodes = {}
            self.links = {}

        def by_attrib(self, attrib_name, attrib_val):
            return [key for key, value in self.nodes.iteritems() if value.get(attrib_name, None) == attrib_val ]

        def check_link(self, from_id, to_id):
            return self.links.get(from_id, {}).get(to_id, False)

    def __make_graph(self, process_tree):
        res = CompileService.Graph()
        for node in process_tree.xpath('//node'):
            d = {}
            for c in node.getchildren():
                if c.tag != "links":
                    d[c.tag] = c.text
            res.nodes[d.get('id')] = d
            for link in node.xpath('links/link'):
                if not res.links.has_key(d.get('id')):
                    res.links[d.get('id')] = {}
                res.links[d.get('id')][link.xpath('to')[0].text] = True
        return res

    def __execute(self, process_src, params, steps):
        graph = self.__make_graph(etree.fromstring(process_src))
        starts = [graph.nodes[n] for n in graph.by_attrib('type', 'takeoff')]
        if not len(starts):
            return ""
        start = random.choice(starts)
        try:
            self.__step_node(graph, start, params, steps)
        except Exception, e:
            params['logger_res'] = params.get('logger_res', '') + '|Error:' + e.message
        return params.get('logger_res', '')

    def __step_node(self, graph, node, params, steps):
        if node is None:
            raise Exception('empty node - broken process')
        if steps > self.step_limit:
            raise Exception('too many steps')
        self.by_type[node.get("type", "none")](self, node, params, steps)
        steps += 1
        targets = [k for k, v in graph.links.get(node.get("id"), {}).iteritems()]
        if len(targets) > 1:
            params['logger_res'] = params.get('logger_res', '') + '|Branching beetween:' + ','.join(targets)
        for t in targets:
            self.__step_node(graph, graph.nodes[t], params, steps)

    def __include(self, node, params, steps):
        file = max(models.FileVersion.all().filter('name =', node["process"]).filter('type =', models.FileType.all().filter('extension =', 'fpml')[0]), key=lambda x: x.updated)
        self.__execute(file.content, params, steps)