# coding=UTF-8
from django.http import Http404
from web import models
from web.services import project
from web.util import CommonUtil
from google.appengine.ext import db

__author__ = 'IoS'

class FileService(object):
    def try_get_master_file(self, project, category, name):
        return self.try_get_file(project, "master", category, name)

    def try_get_file(self, project, tag, category, name):
        ps = self.get_last_project_state(project, tag)
        states = models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", ps))
        for fv in models.FileVersion.select_related([fs.file_version for fs in states]):
            if fv.category == category and fv.name + "." + fv.type.extension == name:
                return fv
        raise Http404

    def try_get_master_path(self, project, path):
        return self.try_get_path(project, "master", path)

    def try_get_path(self, project, tag, path):
        res = []
        ps = self.get_last_project_state(project, tag)
        states = models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", ps))
        for fv in models.FileVersion.select_related([fs.file_version for fs in states]):
            if fv.category == path or fv.category.startswith(path):
                res.append(fv)
        if len(res):
            return res
        raise Http404

    def get_project_files(self, project, account = None):
        return self.get_last_master_project_state_files(project, account)

    def get_last_master_project_state_files(self, project, account = None):
        return self.get_last_project_state_files(project, "master", account)

    def get_last_project_state(self, project, tag):
        ps = CommonUtil.query_to_list(models.ProjectState.all().filter("project =", project).filter("tag =", tag).order("-version"))
        if not len(ps):
            return None
        return models.ProjectState.select_related([ps[0]])[0]

    def get_last_project_state_files(self, project, tag, account = None):
        p = self.get_last_project_state(project, tag)
        if p is None:
            return []
        return self.get_project_state_files(p, account)

    def get_project_state_files(self, project_state, account = None):
        if account is None:
            if not project_state.project.public:
                return []
        else:
            if not project.ProjectService().check_read_rights(account, project_state.project):
                return []
        res = []

        states = models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", project_state))
        for fv in models.FileVersion.select_related([fs.file_version for fs in states]):
            res.append(fv)
        return res

    def find_files_in_public(self, account, part, type = None):
        #TODO
        # all projects with reading, writing, configuring or owning user
        # + all project with public = true
        pass

    def find_files_in_project(self, account, part, project, type = None):
        ps = CommonUtil.query_to_list(models.ProjectState.all().filter("project =", project).filter("tag =", "master").order("-version"))
        if not len(ps):
            return []
        return self.find_files_in_project_state(account, part, ps, type)

    def find_files_in_all_project_states(self, account, part, project, type = None):
        return self.find_files_in_project_states(account, part, models.ProjectState.all().filter("project =", project), type)

    def find_files_in_project_states(self, account, part, project_states, type = None):
        res = []
        for p in project_states:
            res += self.find_files_in_project_state(account, part, p, type)
        return res

    def find_files_in_project_state(self, account, part, project_state, type = None):
        # the lowest method
        pt = project_state.project
        if pt is None or not project.ProjectService().check_read_rights(account, pt):
            return []
        res = []

        states = models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", project_state))
        for fv in models.FileVersion.select_related([fs.file_version for fs in states]):
            res.append(fv)
        return res

    def tag_state(self, state, tag, comment):
        new_state = models.ProjectState(version = state.version + 1, previous_state = state, tag = tag, comment = comment, project = state.project)
        new_state = new_state.put()
        for fvs in models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", state)):
            models.FileVersionState(project_state = new_state, file_version = fvs.file_version).put()
        return models.ProjectState.get(new_state)

    def move_state(self, old_state, project, tag, comment):
        new_state = models.ProjectState(previous_state = old_state, tag = tag, comment = comment, project = project)
        if old_state is not None:
            new_state.version = old_state.version + 1
        new_state.put()
        if old_state is not None:
            for fvs in models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", old_state)):
                models.FileVersionState(project_state = new_state, file_version = fvs.file_version).put()
        return new_state

    def delete_state(self, state):
        for fvs in models.FileVersionState.all().filter("project_state =", state):
            fvs.delete()
        state.delete()

    def add_file_to_state(self, state, file_version):
        for fvs in models.FileVersionState.select_related(models.FileVersionState.all().filter("project_state =", state)):
            if file_version.name == fvs.file_version.name and file_version.type.key().id() == fvs.file_version.type.key().id():
                fvs.delete()
        models.FileVersionState(project_state = state, file_version = file_version).put()

    def delete_file_from_state(self, state, file_version):
        to_delete = models.FileVersionState.all().filter("project_state =", state).filter("file_version =", file_version)
        for o in to_delete:
            o.delete()

    def edit_file_to_state(self, state, old_file_version, new_file_version):
        self.delete_file_from_state(state, old_file_version)
        self.add_file_to_state(state, new_file_version)

    def get_level(self, path, nodes):
        return [{"dir": len(node["part"]) > 0,
                 "name": node["part"].split("/")[0] if len(node["part"]) else node["node"].name + "." + node["node"].type.extension
                }
                    for node in [{"part": (node.category.replace(path, "", 1).replace("/", "", 1) if node.category.replace(path, "", 1).startswith("/") else node.category.replace(path, "", 1)),
                                  "node": node
                                 }
                                    for node in nodes]]

    def apply_modification(self, file, modification):
        if int(modification["file"]["id"]) != file.key().id():
            raise Exception("Inconsistent modification")
        res = str(file.content).decode('utf-8')
        if modification["type"] == "insert":
            res = res[:int(modification["start"])] + modification["newValue"] + res[int(modification["start"]):]
        if modification["type"] == "replace":
            res = res[:int(modification["start"])] + modification["newValue"] + res[int(modification["start"]) + int(modification["length"]):]
        if modification["type"] == "delete":
            res = res[:int(modification["start"])] + res[int(modification["start"]) + int(modification["length"]):]
        return db.Blob(res.encode('utf-8'))

    def clean_file(self, file_dict):
        del file_dict["content"]
        return file_dict

    def apply_file_rights(self, file_dict, account, project_obj):
        file_dict["is_writable"] = project.ProjectService().check_write_rights(account, project_obj)
        return file_dict