# coding=UTF-8
from django.http import Http404
from web import models
from web.util import CommonUtil

__author__ = 'IoS'

class ProjectService(object):
    def raise_not_found(self):
        raise Http404
    
    def get_project(self, project_name):
        ps = CommonUtil.query_to_list(models.Project.all().filter("link_name =", project_name))
        return models.Project.select_related([ps[0]])[0] if len(ps) else self.raise_not_found()

    def get_owned_projects(self, account):
        return models.Project.select_related(CommonUtil.query_to_list(models.Project.all().filter("owner =", account)))

    def get_writeable_projects(self, account):
        return models.Project.select_related([item.project for item in models.Participant.select_related(models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.WRITE))] +\
                [item.project for item in models.Participant.select_related(models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.CONFIGURE))])

    def get_readable_projects(self, account):
        return models.Project.select_related([item.project for item in models.Participant.select_related(models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.READ))])

    def get_invited_projects(self, account):
        return models.Project.select_related([item.project for item in models.Participant.select_related(models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.INVITED))])

    def find_projects(self, name, type):
        f = models.Project.all()
        if name is not None and len(name):
            f = f.search(name)
        if type is not None and len(type):
            f = f.filter('type =', models.ProjectType.get_by_id(int(type)))
        return models.Project.select_related(CommonUtil.query_to_list(f))

    def check_owner_rights(self, account, project):
        return project.owner.key().id() == account.key().id()

    def check_configure_rights(self, account, project):
        return self.check_owner_rights(account, project) or models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.CONFIGURE).filter("project =", project).count() > 0

    def check_write_rights(self, account, project):
        return self.check_configure_rights(account, project) or models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.WRITE).filter("project =", project).count() > 0

    def check_read_rights(self, account, project):
        return project.public or self.check_write_rights(account, project) or models.Participant.all().filter("who =", account).filter("allowed =", models.Participant.READ).filter("project =", project).count() > 0
